/************************************************************************
  GPRMODEL.C:  One-Dimensional Full Waveform Forward Modeling of
			  Ground Penetrating Radar Data
  by M.H.Powers, S.K.Duke, A.C.Huffman III, and G.R.Olhoeft

			 U. S. Geological Survey 
			 P.O. Box 25046 DFC MS964
			 Denver, Colorado  80225
			 (303)-236-1302 or 1413 or 1349              
			 
  based, in part, on:

  Duke, S. K., 1990, Calibration of ground penetrating radar and calculation
	 of attenuation and dielectric permittivity versus depth: MSc Thesis,
	 Dept. of Geophysics, Colorado School of Mines, Golden, 236p.

**************************************************************************/
#include <stdio.h>
#include <string.h>
#include <math.h>
#include <hpgl.h>       /* for graphics functions, full reference follows: */
/* Olhoeft, G.R., 1990, libhpgl.lib 32-bit protected mode extended graphics library:
	   The Austin Code Works, Austin, Tx, version 8.0.                    */
#include <conio.h>
#include <stdlib.h>
#include <windef.h>     /* for window functions, full reference follows: */
/* Pinson, J.,1991, Designing Screen Interfaces in C: Englewood Cliffs, NJ,
	   Yourdon Press, 267p.                                               */
#undef  NULL
#define NULL 0
#define N 512           /* Radix 2 */
#define FRAMP 9         /* Number of points in Hanning ramp for frequency filter */
#define W 23            /* Number of points in range-gain envelope smoother */
#define eo 8.854e-12    /* Dielectric permittivity of free space */
#define uo PI*4.e-7     /* Magnetic permeability of free space */
#define Vc 0.2998       /* Velocity of EM in free space in m/ns */
#define LITER 50        /* number of points to ignore at beginning of gain scan */
#define SQR(a) (sqrarg=(a),(sqrarg)*(sqrarg))
#define SWAP(a,b) tempr=(a);(a)=(b);(b)=tempr
#define Cradd(a,b) ((a.r)+(b.r))  /* complex math */
#define Ciadd(a,b) ((a.i)+(b.i))
#define Crmul(a,b) ((a.r)*(b.r))-((a.i)*(b.i))
#define Cimul(a,b) ((a.i)*(b.r))+((a.r)*(b.i))
#define M 11            /* Maximum number of modeling layers + 1 */
#define TRUE 1
#define FALSE 0
#define ESC 27
#define SPACE 32
#define ALT_S 31
#define ALT_P 25
#define ALT_V 47
#define ALT_O 24
#define ALT_R 19
#define ALT_F 33
#define ALT_T 20
#define C_TOP 1    /* Top Window Color */
#define C_ALT 14   /* <ALT> Color */
#define W_ALT 1    /* <ALT> Window Color */
#define C_MAIN 11  /* Main Window Color */
#define C_KEY 7    /* Key Window Color */
#define C_HELP 2   /* Help Window Color */
#define C_WIN_BOR 15    /* Window Border Color */
#define TOGGLE 12  /* Toggle color */
#define FNF 12     /* File not found color */
#define W_SAVE 4   /* Save window color */
#define C_LIST 12  /* File list color */
#undef BOLD        /* Bold already defined in mydef.h */
#define BOLD 14    /* Bold color for one key press */
#define RESTORE 7  /* Restored cursor color */
#define DOUBLE 1   /* double border window */
#define SINGLE 2   /* single border window */
typedef struct {double r,i;} complex;
/******************* Subroutines in order of listing *********************/
void intro(int g_mode); /* displays introductory screens */
void window_setup();    /* Set-up 4 windows */
void key_win(int option);         /* Display window showing valid keys to press */
void ALT_win(int option);         /* Display window showing Alt+ options */
int import_or_create();           /* Ask to import or create modeling file */
int import_fname(int option);     /* If import file, ask user name of the file */
int desire_change_param();        /* Ask if parameter wants to be changed */
int which_param();      /* If parameter wants to be changed, enter which parameter */
int which_layer();      /* If layer parameter wants to be changed, enter which layer */
int input_model(int l,int flag,int p); /* Enter layer parameters */
int which_layer_param(int a5);         /* If change one layer parameter, choose which one */
int new_layer_param(int a5,int a7);    /* If change one layer parameter, enter new value */
int input_plt_title();  /* If create file, enter plot title */
int num_layers();       /* If create file, enter number of layers to create */
int create_model();     /* If create file, enter layer parameters */
int desire_raw_data();  /* Ask if raw field data should be imported */
int import_raw_rg_tc(); /* Import raw field and range gain files */
void save_file(int option);  /* Prompt to save file */
int check_save();       /* Check if an electrical parameter has been changed */
void read_file(FILE *p_file);/* read previously created processing file */
void list_file();       /* List file on screen */
void file_not_good(int option);   /* File not found window */
int line_editor(int ans_col,int ans_row,int max_column,int color, \
	char *answer_str,int isfile,int isnumber);  /* data input routine */
void plot_info();       /* information windows */
void vary_toggle();     /* turn on & off interactive mode */
void options();         /* turn on & off optional calculations */
int check_file(FILE *p_file);     /* is requested .MOD file present? */
void check_quit();      /* If F10 is pressed, exit to DOS window is prompted */
void restore_cur();     /* Restore cursor to original size and clear screen */
void plot_setup(char *title,int pltfile,int g_mode);   /* draw plot bkgrnd */
void plot_layer();      /* draw layer model on right side of screen */
int increment_layer(double *dis_tmp);   /* interactive param change */
void plot_scan(double off,double *data,int color);    /* plots data */
void calc_att_dis(double *data,double delf);/* calculates layer attenuation */
void ray_trace(int rl,int im,double real,double imag);/* calc effects thru model */
void mult_trace(int rl,int im);   /* optionally calc effects of multiples */
void ray_model();  /* sum reflections to get model data (inverse FFT back to time) */
void mult_model(); /* inverse FFT and sum multiples into model */
complex Cdiv(complex a,complex b);     /* performs complex division */
complex Cprop(complex e,double con,double w);    /* finds k.r & k.i for a layer */
complex Catt(complex c,complex k,double dist);   /* find attenuation due to dist */
complex Cref(complex c,complex k1,complex k2);   /* find effect of boundary reflection */
complex Ctran(complex c,complex k1,complex k2);  /* find effect of boundary transmission */
void resample(int option,double *sdata,double old_samp,double new_samp,int g_mode);
void import_field(int option);    /* input .CAL file */
void import_tran(int option);     /* input .TRN file */
int display_gain(int g_mode);     /* calculate & remove range-gain using .RG file */
void smooth_data(double *data, int n, int w);    /* smooths by window w */
void remove_gain(double *maxf, double *minf, int g_mode);  /* removes gain */
void eliminate_field(int g_mode); /* zero out direct arrivals */
void time_calibrate(int g_mode);  /* find f_sample from .TC data */
void derivative(double *data,int width,int n);   /* return deriv smoothed by width */
void pick(double *data, int n, int np, int hilo, int *pkpnts);  /* find np hi or lo pnts */
int count(double *data, int n);        /* count # of minima */
double mean(int *pkpnts, int cnt);     /* return mean of cnt values in pkpnts */
double stddev(int *pkpnts, int cnt, double mn);  /* return std deviation */
void graph_setup(char *text,int g_mode);     /*  Setup gain display and stack graphs */
/************************************************************************/
/* The next 4 subroutines do FFT and spline from: ***********************/
/* Press, W.H., Flannery, B.P., Teukolsky, S.A., Vetterling, W.T., 1988, 
	   Numerical Recipes in C, Cambridge Univ. Press, Cambridge, 735p.  */
void realft(double *data,int n,int isign);        
void four1(double *data,int nn,int isign);  
void spline(double *x,double *y,int n,double yp1,double ypn,double *y2); 
void splint(double *xa,double *ya,double *y2a,int n,double x,double *y);
/************ Global variables********************************************/
static double sqrarg;
unsigned char craw[N+1];
char fproc[80],ftran[80],title[80],ffield[80],plot_file[80]="CRT";
char tittmp[80],ftmp[80],ttmp[80],fraw[80],fgain[80],ftime[80];
int lnum,delay[M],mode=FALSE,l,p,aa1=FALSE,aa2=FALSE,aa3=FALSE;
int wtitle,work,help,keys,list,change,quit,wexit,cr_flag=0;
int save,resam,option,vary,wfield,trans,wplot,overw,g_mode;
double con[M],dis[M],d[M],dd[M],model[N+1],sample=0.0,first,last,offset,ratio;
double reflect[M][N+1],mult[M][N+1],dtdata_in[N+2],c_ratio=1.0,vinc=0.0,min_cr;
double dtdata[N+2],dfield[N+2],dfield_in[N+2],parm[4*M],t_sample,f_sample=0.0;
complex e[M],k[M],att_dis[M],ray[M];

extern struct screen_structure scr;
/************************************************************************/
/************************************************************************/
main(int argc, char *argv[])
{
  char lab[100],att_lab[100];
  int q,rem,a1,a2,a3,a4,a5,a6,a7,a8,a9,a10,a11,a12,a13;
  int i,j,rl,h,im,loop,plotfield,pltfile;
  double delf,w,mn,mx,powdif=0.,maxpow,pidelf,dif,temp_ratio;
  double data[N+1],dis_tmp[M],clr_model[N+1];
  FILE *p_file,*t_file;
  e[0].r=1.;   /** Air parameters **/
  d[0]=dd[0]=e[0].i=con[0]=0.;
  delay[0]=0;

  if( (argc<2) || (argv[1]=="xx") )
  {
    printf("The calling sequence for this program is: gprmodel mode\n");
    printf("where:\n");
    printf("EGA hi-res           (640x350x16-colors)  --- mode =   16\n");
    printf("VGA                  (640x480x16)         --- mode =   18\n");
    printf("MCGA                 (320x200x256)        --- mode =   19\n");
    printf("8514/A               (1024x768x256)       --- mode = 8514\n");
    printf("ATI VGA Wonder       (800x600x16)         --- mode =   84\n");
    printf("HP D1180A/Paradise   (800x600x16)         --- mode =   88\n");
    printf("HP D1180A/Paradise   (640x480x256)        --- mode =   95\n");
    printf("ATI VGA Wonder       (640x480x256)        --- mode =   98\n");
    printf("ATI VGA Wonder       (800x600x256)        --- mode =   99\n");
    printf("ATI VGA Wonder       (1024x768x16)        --- mode =  101\n");
    printf("Tseng 4000 chipset   (1024x768x256)       --- mode =   56\n");
    printf("Hercules GB1024      (1024x768x256)       --- mode = 1024\n");
    exit(0);
  }
  begin();
  g_mode=atoi(argv[1]);          /* Initialize graphics mode */
  intro(g_mode);

  do
  {
    if(work==0) window_setup();  /* Clear screen and set-up  4 windows */
    else win_cls(work);
    a1=import_or_create();                     /* Create or Import file */
    if(a1==ESC) continue;
    if(a1)                       /* If user wants to create a file */
    {
	 strcpy(fproc,"UNTITLED");  /* Creation file name */
	 strcpy(ftran,"UNTITLED");  /* Transmission wavelet file name */
	 strcpy(ffield,"UNTITLED"); /* Field file name */
	 for(i=1; i<7; i++)
	 {
	   if(i==1)                 /* Enter transmission file name */
	   {

		do
		{
		  a2=import_fname(0);            /* Enter name of file to Import */
		  if(a2==ESC) break;
						   /* If Import file cannot be found */
		  if((t_file=fopen(ftran,"rb"))==NULL) file_not_good(4); 
		} while(t_file==NULL);

		if(a2==ESC) 
		{
		  i-=2; 
		  a2=(-1);
		} 
		else 
		{
		  import_tran(0);      /* get transmission data */
		  sample=t_sample;     /* t_sample is the sample rate of .TRN file data */
		  c_ratio=1.0;         /* c_ratio shrinks or expands the .TRN wavelet  */
		}    
	   }      
	   if(i==2)                 /* Enter number of layers */
	   {
		a10=num_layers();
		if(a10==ESC) {i-=2; a10=(-1); ceol(1,5);}
	   }
	   if(i==3)                 /* Enter plot title */
	   {
		a9=input_plt_title();
		if(a9==ESC) {i-=2; a9=(-1); ceol(1,6);}
	   }
	   if(i==4)                 /* Option to import raw field data for gain-removal */
	   {
		a12=desire_raw_data();
		if(a12==ESC) {i-=2; a12=(-1); ceol(1,7);}
	   }
	   if(i==5 && a12==TRUE)    /* If import raw field, import .TC and .RG files */
	   {
		a13=import_raw_rg_tc();
		if(a13==ESC) {i-=2; a13=(-1); ceol(1,8); ceol(1,9); ceol(1,10);}
		else
		{
		  time_calibrate(g_mode);  /* Time calibration here */
		  if (sample!=0.0)          
		  {                    /* set .TRN wavelet to display sample rate */
		    for(h=1;h<=N;h++)  dtdata[h]=dtdata_in[h]; 
		    resample(0,dtdata,t_sample,sample,g_mode); 
		  }
		  rem=display_gain(g_mode);/* Display and remove gain envelope */
		}
		if(rem==ESC) {win_cls(work); i-=2; rem=(-1);}
	   }
	   if(i==6)                 /* Enter modeling parameters */
	   {
		a11=create_model();
		if(a11==ESC) {i-=3; win_cls(work); a11=(-1);}
	   }
	   if(i<0) break;
	 }       /*************************** close for loop ****************/

	 if(i<0) continue;
    }         /*************************** close if(a1) loop *************/

    else      /*************************** Import a file *****************/
    {

	 do
	 {

	   do
	   {
		a2=import_fname(1);              /* Enter name of file to Import */
		if(a2==ESC) break;
		if((p_file=fopen(fproc,"rt"))==NULL) file_not_good(1); 
	   } while(p_file==NULL);

	   if(a2==ESC) break;
	   if(check_file(p_file)) file_not_good(2);
	   else
	   {
		read_file(p_file); /* read Import file */
		import_tran(0);    /* get transmission data */
		if((strcmp(ffield,"UNTITLED"))!=0) 
		{
		  import_field(0);   /* get field data */
		  if(f_sample!=sample) resample(0,dfield,f_sample,sample,g_mode);
		}
		resample(0,dtdata,t_sample,sample/c_ratio,g_mode); 
		break;
	   }
	 } while(TRUE);  /* Loop until a valid file is found */

	 if(a2==ESC) {win_cls(work); continue;}
    }         /******************** end import a file else condition ****/
    /********************************************************************/

    do
    {
	 win_cls(work);             /* Clear main window */
	 q=2;

	 do
	 {
	   q++;
	   switch(q)
	   {
		case 3:                /* Ask if user wants to change a parameter */
		  if(list) win_delete(list);
		  list_file();         /* Display Imported parameters on screen */
		  ALT_win(1);          /* Display ALT options on top of main window */
		  a3=desire_change_param();
		  if(a3==ESC) if(check_save()) save_file(1); /* save if param have changed? */
		  ALT_win(0);          /* Remove ALT options from top of main window */
		  break;
		case 4:                /* Ask what user wants to change */
		  a4=which_param();
		  if(a4!=1 && a4!=ESC) q=2;
		  if(a4==ESC) {q-=2; a4=(-1); win_pop_top(change); ceol(1,1);}
		  break;
		case 5:                /* Enter layer to change */
		  a5=which_layer();
		  if(a5==(-1)) q=2;
		  if(a5==ESC) {q-=2; a5=(-1); win_pop_top(change); ceol(1,1);}
		  break;
		case 6:                /* If user wants to add an extra layer */
		  a7=which_layer_param(a5);
		  if(a7==ESC) {q-=2; a7=(-1); win_pop_top(change); ceol(1,1);}
		  break;
		case 7:                /* Enter new parameter value */
		  a8=new_layer_param(a5,a7);
		  if(a8!=ESC) q=2;
		  if(a8==ESC) {q-=2; a8=(-1); win_pop_top(change); ceol(1,1);}
		  break;
	   }
	 } while(a3==TRUE && a3!=ESC);    /* Loop until a3==FALSE or a3=ESC */

	 if(a3==ESC) continue;

	 /***************** Start modeling **********************************/
	 if(strcmp(ftran,"UNTITLED")==0) /* If .TRN file does not exist */
	 {
	   printf("\a");
	   win_make(13,9,42,2,STD_FRAME,"",WHITE,RED);
	   print(1,1,"Transmitted wavelet signature file does not exist.");
	   print(4,2,"Press <Alt>+T to load file.");
	   win_redraw_all();
	   getch(); 
	   win_delete_top();
	   continue;
	 }
	 /**************** Initialize values ********************************/
	 delf=1./(sample*(double)N*1.e-9);     /* delta frequency */
	 pidelf=2.*PI*delf;
	 mn=mx=0.;
	 if(mode)              /* If toggle vary is ON initialize parameters */
	 {
	   l=p=1;
	   for(i=1;i<lnum;i++) dis_tmp[i]=dis[i];
	   for(i=1;i<lnum;i++) d[i]=dd[i]=d[i-1]+dis[i];
	 }
	 for(i=1;i<=N;i++) model[i]=0.;   /* zero model data array */
	 if(aa3)               /* Intialize maximum power */
	 {
	   maxpow=1e-5;
	   for(i=(int)first;i<=(int)last;i++)
	   {
		if((i-(int)offset)<1 || (i-(int)offset)>511) continue;
		maxpow+=SQR(dfield[i-(int)offset]);
	   }
	 }
	 if(strcmp(ffield,"UNTITLED")!=0) /* If field file exists find window limits */
	 {
	   plotfield=1;
	   mn=9999.;                /* Determine window limits */
	   mx=(-9999.);
	   for(i=(int)first;i<=(int)last;i++)
	   {
		if((i-(int)offset)<1 || (i-(int)offset)>511) continue;
		mn=MIN(mn,dfield[i-(int)offset]);
		mx=MAX(mx,dfield[i-(int)offset]);
	   }
	   dif=mx-mn;
	   mn-=(dif*.2);
	   mx+=(dif*.2);
	 }
	 else                       /* No field file to plot */
	 {
	   plotfield=0;
	   aa3=FALSE;
	 }
	 if(strcmp(plot_file,"CRT")!=0) pltfile=1; /* plot is CRT or .PLT? */
	 else pltfile=0;
	 /********************** Graphics-Plot setup ************************/
	 plot_setup(title,pltfile,g_mode);
	 /***************** Plot layer Model ********************************/
	 plot_layer();
	 clip();

	 do
	 {
	   for(i=1;i<=N;i++) clr_model[i]=model[i];
	   /********************* Begin processing **************************/
	   for(i=1;i<=N;i++) data[i]=dtdata[i];
	   realft(data,N/2,1);      /* Forward FFT data */
	   if(aa2) calc_att_dis(data,delf);
	   for(i=1;i<=N/2;i++)
	   {
		rl=2*i-1;
		im=2*i;
		w=pidelf*(double)(i);
		/******** Calculate propogation parameters for each layer*******/
		for(j=0;j<=lnum;j++) k[j]=Cprop(e[j],con[j],w);
		/************************* Ray Trace****************************/
		ray_trace(rl,im,data[rl],data[im]);
		/************* Calculate 3 multiples if desired ****************/
		if(aa1) mult_trace(rl,im);
	   }
	   /*************** Add reflections to get model ********************/
	   ray_model();
	   if(aa1) mult_model();
	   /* If no field data, find window mn/mx from forward model ********/
	   if(plotfield==FALSE)
	   {
		viewport(0.5,99.5,8.5,94.5);
		window(first,last,mn,mx);
		plot_scan(0.,clr_model,0);
		mn=1e32;               /* Determine window limits */
		mx=(-1e32);
		for(i=first;i<=last;i++)
		{
		  mn=MIN(mn,model[i]);
		  mx=MAX(mx,model[i]);
		}
		dif=mx-mn;
		mn-=(dif*.2);
		mx+=(dif*.2);
		if(abs(dif)<1e-30) {mn=(-10.); mx=10.;}
	   }
	   /******* Clear model & Plot field scan ***************************/
	   viewport(0.5,99.5,8.5,94.5);
	   window(first,last,mn,mx);
	   if(pltfile==FALSE) plot_scan(0.,clr_model,0);
	   if(pltfile) linetype(2);
	   if(plotfield) 
	   {
		plot_scan((offset-vinc),dfield,0);
		plot_scan(offset,dfield,12); /* plot field scan if it was imported */
	   }
	   /******** Plot forward model scan ********************************/
	   if(pltfile) linetype(0);
	   plot_scan(0.,model,14);
	   window(0.,100.,0.,100.);
	   /************* Plot layer attenuation if desired *****************/
	   if(aa2)
	   {
		lorg(8); csize(.9); pen(0);
		label(95.,8.5,att_lab);
		strcpy(att_lab,"\0");
		for(i=1;i<=lnum;i++)
		{
		  sprintf(lab,"a%d=%4.3f ",i,att_dis[i].r);
		  strcat(att_lab,lab);
		}
		strcat(att_lab,"Nep/m");
		pen(15); /* white */
		label(95.,8.5,att_lab);
	   }
	   /* Compute power difference between model and field if desired ***/
	   if(aa3)
	   {
		lorg(8); pen(0); csize(1.);
		sprintf(lab,"POW = %5.4f",powdif);
		label(95.,4.,lab);
		powdif=0.;
		for(i=(int)first;i<=(int)last;i++)
		{
		  if((i-(int)offset)<1 || (i-(int)offset)>511) continue;
		  powdif+=SQR(model[i]-dfield[i-(int)offset]);
		}
		powdif*=(100./maxpow);
		pen(11); /* cyan */
		sprintf(lab,"POW = %5.4f",powdif);
		label(95.,4.,lab);
	   }
	   /************* Change parameters in graphics mode if desired *****/
	   if(mode) 
	   { 
		loop=increment_layer(dis_tmp);
		if(cr_flag) resample(0,dtdata,t_sample,sample/c_ratio,g_mode);
	   }
	 } while(loop!=ESC && mode==TRUE);

	 /************************ end of plot layer model do loop **********/

	 if(mode==FALSE) getch();
	 plotter_off();
	 /***************** End modeling ************************************/
	 win_redraw_all();
    } while (a3!=ESC);  

    /******* Loop until ESC returns to beginning of questioning **********/

  } while(TRUE);   /****** Continue to loop until F10 is pressed *********/

  exit(0);
}
/****************** end of main ******************************************/

/*************************************************************************/
void intro(int g_mode)           /* displays introductory screens */
{
  int key,i,ii,j;
  double w, pi180, diff;
  pi180 = PI/180.;

  plotter_is(g_mode,"CRT");      /* setup graphics mode */
  viewport(0.,133.,0.,100.);     /* use full screen */
  window(0.,100.,0.,100.);       /* scale viewport to window */
  pen(15); lorg(5); csize(1.);   /* white, labels centered, char size 1 */
  label(60.,5.,"Press a key to continue.");
  csize(4.);                     /* char size 4 */
  label(50.,90.,"GPR");
  csize(3.);
  label(50.,70.,"FORWARD MODELING");
  j = 1;

  do                             /* animation sequence at beginning of program */
  {
    if (j==1)
    {
	 pen(0);                    /* black */
	 for (i=0; i<361; i += 2)
	 {
	   w = (double)(i)*pi180;
	   plot(55.+cos(w),53.+sin(w),-1-(i==0));
	 }
	 pen(15); /* white */
	 plot(0.,50.,-2);
	 plot(100.,50.,-1);
	 pen(14); /* yellow */
	 for (i=0; i<181; i++)
	 {
	   w = (double)(i)*pi180;
	   plot(25.+5.*cos(w),55.-5.*sin(w),-1-(i==0));
	 }
	 plot(30.,55.,-1);
	 for (i=0; i<181; i++)
	 {
	   w = (double)(i)*pi180;
	   plot(55.+5.*cos(w),55.-5.*sin(w),-1-(i==0));
	 }
	 plot(60.,55.,-1);
	 for (i=0; i<361; i++)
	 {
	   w = (double)(i)*pi180;
	   plot(25.+5.*cos(w),6.+5.*sin(w),-1-(i==0));
	 }
	 pen(12); /* red */
	 for (i=0; i<361; i += 2)
	 {
	   w = (double)(i)*pi180;
	   plot(25.+cos(w),53.+sin(w),-1-(i==0));
	 }
    } 
    pen(0);
    if (j==3)
    { 
	 for (i=0; i<361; i += 2)
	 {
	   w = (double)(i)*pi180;
	   plot(25.+cos(w),53.+sin(w),-1-(i==0));
	 }
    }
    diff = (double)(7*(j-1));
    for (i=10; i<170; i += 2)
    {
	 w = (double)(i)*pi180;
	 plot(25.+diff*cos(w),55.-diff*sin(w),-1-(i==10));
    }
    if (j==14)
    {
	 pen(12); /* red */
	 for (i=0; i<361; i += 2)
	 {
	   w = (double)(i)*pi180;
	   plot(55.+cos(w),53.+sin(w),-1-(i==0));
	 }
    }
    pen(9); /* blue */
    diff = (double)(7*j);
    for (i=10; i<170; i += 2)
    {
	 w = (double)(i)*pi180;
	 plot(25.+diff*cos(w),55.-diff*sin(w),-1-(i==10));
    }
    j = (j++ % 15) + 1;
    pen(0);
    diff = (double)(7*(j-7));
    for (i=10; i<170; i += 2)
    {
	 w = (double)(i)*pi180;
	 plot(25.+diff*cos(w),7.+diff*sin(w),-1-(i==10));
    }
    pen((j>7)-(j==15));
    diff = (double)(7*(j-6));
    for (i=10; i<170; i += 2)
    {
	 w = (double)(i)*pi180;
	 plot(25.+diff*cos(w),7.+diff*sin(w),-1-(i==10));
    }
    if((key=kbhit())!=0) break;  /* wait for key hit */
  } while(TRUE);

  getch();
  plotter_off();                 /* return to text mode */
  set_mode(3);                   /* Text mode 80 column */
  win_make(1,1,78,24,DBL_FRAME,"",15,15);
  print(18,1,"UNITED STATES DEPARTMENT OF THE INTERIOR");
  print(29,2,"GEOLOGICAL SURVEY");
  print(8,4,"GPRMODEL: One-Dimensional Full Waveform Forward Modeling of");
  print(23,5,"Ground Penetrating Radar Data");
  print(38,6,"by");
  print(24,8,"Michael H. Powers, Steven K. Duke,");
  print(21,9,"Arlie C. Huffman III, and Gary R. Olhoeft");
  print(26,11,"Open File Report 92-532");
  print(30,13,"September 1992");
  print(30,15,"Branch of Geophysics");
  print(26,16,"U.S. Geological Survey MS964");
  print(31,17,"P.O. Box 25046 DFC");
  print(26,18,"Denver, Colorado  80225-0046");
  print(26,19,"(303) 236-1302, -1212");
  print(5,21,"Press a key to continue.");
  cursor(NO_CURSOR);
  getch();
  win_cls(1);
  win_redraw_all();
  print(37,1,"NOTICE");
  print(4,3,"U.S. Geological Survey GPR forward modeling computer program.");
  print(4,4,"The program was built using the Intel 386/486 C Code Builder version");
  print(4,5,"1.1a compiler.  The code was written to run under MS-DOS 3.+ on");
  print(4,6,"80386/80387 or 80486 computers.  The text window functions are from:");
  print(4,7,"Pinson, J., 1991, Designing Screen Interfaces in C: Englewood Cliffs,");
  print(4,8,"      NJ, Yourdon Press, 267p.");
  print(4,10,"This program is based in part on the thesis research described in:");
  print(4,11,"Duke, S.K., 1990, Calibration of ground penetrating radar and");
  print(4,12,"     calculation of attenuation and dielectric permittivity versus depth,");
  print(4,13,"     Master's thesis, Colorado School of Mines, Golden, CO, 236 pp.");
  print(4,15,"Although this program has been used by the U.S. Geological Survey,");
  print(4,16,"no warranty, expressed or implied, is made by the USGS as to the");
  print(4,17,"accuracy and functioning of the program and related program material");
  print(4,18,"nor shall the fact of distribution constitute any such warranty,");
  print(4,19,"and no responsibility is assumed by the USGS in connection herewith.");
  print(4,21,"Press any key to continue.");
  cursor(NO_CURSOR);
  getch();
  win_cls(1);
  win_redraw_all();
  print(34,1,"INTRODUCTION");
  print(4,3,"     This program is designed to aid in the determination of subsurface");
  print(4,4,"electrical properties, and demonstrate how these properties alter the");
  print(4,5,"signature of a radar scan.  A file containing a calibrated, transmitted");
  print(4,6,"wavelet signature at time zero [stored as 513 doubles, where the last one");
  print(4,7,"is the sample interval in nanoseconds] is the minimum requirement for");
  print(4,8,"this program to begin.  The amplitude and time-calibrated transmitted");
  print(4,9,"wavelet is traced through layered media to compute GPR forward models.");
  print(4,10,"Also, files containing raw field data [1 scan], range gain calibration");
  print(4,11,"records and time-calibration records [all stored as 512 bytes] may be");
  print(4,12,"imported for the purpose of comparing the forward model to amplitude and");
  print(4,13,"time-calibrated field data.  The range gain records are used to remove");
  print(4,14,"the range gain applied in the field.  The time calibration records are");
  print(4,15,"are used to compute the field scan sample interval, which is used to");
  print(4,16,"compute frequency dependent properties in forward modeling.  The");
  print(4,17,"transmitted wavelet is automatically resampled to match the field scan");
  print(4,18,"sample interval.  The transmitted wavelet may be stretched or squeezed");
  print(4,19,"to simulate antenna-surface coupling changes.");
  print(4,23,"Press a key to continue.");
  cursor(NO_CURSOR);
  getch();
  win_cls(1);
  win_redraw_all();
  print(34,1,"NOMENCLATURE");
  print(4,3,"Symbol      SI Units         Name");

  print(4,4,"-------------------------------------------------------------------");
  print(4,5,"K'            -          real part of complex relative permittivity");
  print(4,6,"K''           -          imaginary part of complex relative permittivity");
  print(4,7,"         millisiemens/m electrical conductivity");
  print(4,8,"Thickness    meters      layer thickness");
  print(4,9,"t        nanoseconds    radar scan sample rate");
  print(4,9,"Cr            -          coupling ratio (pulse broadening)");
  print(4,21,"Press a key to continue.");
  cursor(NO_CURSOR);
  getch();
  win_delete_top();
}

/*************************************************************************/
void window_setup()  /* Set-up 4 windows */
{
  int i;

  wtitle=win_make(1,1,80,1,NO_FRAME,"",WHITE,LBLUE);
  print(3,1,"GPR Forward Model Ver. 1.0 by M.H. Powers and others, USGS - Denver");
  work=win_make(2,3,78,16,DBL_FRAME,"",WHITE,C_MAIN);
  help=win_make(2,21,78,3,DBL_FRAME," HELP ",WHITE,LGREEN);
  keys=win_make(1,25,80,1,NO_FRAME,"",WHITE,LMAG);
  key_win(1);
}

/*************************************************************************/
void key_win(int option)  /* Display window showing valid keys to press */
{
  win_cls(keys);
  scr.current=LMAG;
  if(option==0) 
  {
    win_print(keys,2,1,"Press a Key.");
  }
  if(option==1 || option==2)
  {
    win_print(keys,2,1,"<Esc>-BACKUP");
    win_print(keys,70,1,"<F10>-QUIT");
  }
  if(option==1) 
  {
    win_print(keys,16,1,"  ");
  }
  if(option==2) 
  {
    win_print(keys,15,1,"  <Ins> <Del> <Home> <End> <Backspace> ");
  }
  if(option==3) 
  {
    win_print(keys,2,1,"<Y> <y>  <N> <n>");
  }
  if(option==4) 
  {
    win_print(keys,2,1,"<Esc> <Return> <M> <m> <L> <l> <P> <p>");
  }
  if(option==5) 
  {
    win_print(keys,2,1,"<ESC> <Return>");
  }
  win_redraw_all();
}

/*************************************************************************/
void ALT_win(int option)    /* Display window showing Alt+ options */
{
    scr.current=WHITE;
    if(option)
    {
	 win_print(work,14,1,"ave ");
	 win_print(work,21,1,"ran ");
	 win_print(work,28,1,"ield ");
	 win_print(work,36,1,"lot ");
	 win_print(work,43,1,"ary-");
	 win_print(work,53,1,"Coupling ");
	 win_print(work,63,1,"atio ");
	 win_print(work,71,1,"ptions ");
	 scr.current=YELLOW;
	 win_print(work,3,1," <Alt>+ ");
	 win_print(work,13,1,"S");
	 win_print(work,20,1,"T");
	 win_print(work,27,1,"F");
	 win_print(work,35,1,"P");
	 win_print(work,42,1,"V");
	 win_print(work,62,1,"R");
	 win_print(work,70,1,"O");
	 if(mode) {win_print(work,47,1,"ON");}
	 if(mode==FALSE) {win_print(work,47,1,"OFF");}
    }
    else
    {
	 win_ceol(2,1,1);
	 win_redraw_all();
    }
}

/*************************************************************************/
int import_or_create()      /* Ask to import or create modeling file */
{
  int key,a1=FALSE;

  win_pop_top(work);
  cursor(NO_CURSOR);
  scr.current=C_MAIN;
  print(16,3,"Data file ==>   Import    Create ");
  scr.current=BOLD;
  print(42,3,"C");
  scr.current=scr.inverse;
  print(31,3,"<Import>");
  scr.current=LGREEN;
  win_cls(help);
  win_print(help,3,2,"Import modeling parameter file for GPR forward modeling.");
  key_win(1);

  while((key=getch())!=RETURN && key!=ESC)
  {
    if(key==0 || key==0xE0) key=getch();
    if(key==F10) break;
    scr.current=C_MAIN;
    if(key==73 || key==105) 
    {
	 a1=FALSE;
	 print(32,3,"Import    Create ");
	 scr.current=BOLD;
	 print(42,3,"C");
	 scr.current=scr.inverse;
	 print(31,3,"<Import>");
	 break;
    }
    if(key==67 || key==99)
    {
	 a1=TRUE;
	 print(16,3,"Data file ==>   Import    Create ");
	 scr.current=BOLD; 
	 print(32,3,"I");
	 scr.current=scr.inverse;
	 print(41,3,"<Create>");
	 break;
    }
    if(key==LEFT || key==RIGHT)
    {
	 if(key==LEFT ) if(a1==TRUE) a1=FALSE;
	 if(key==RIGHT) if(a1==FALSE) a1=TRUE;
	 print(16,3,"Data file ==>   Import    Create ");
	 scr.current=BOLD; 
	 print(42,3,"C");
	 print(32,3,"I");
	 scr.current=scr.inverse;
	 if(a1==TRUE)
	 {
	   print(41,3,"<Create>");
	   scr.current=LGREEN;
	   win_cls(help);
	   win_print(help,3,1,"Create modeling parameter file for GPR forward modeling.");
	   win_print(help,3,2,"Values may be later altered in the \
		   program to improve match to field data.");
	 }
	 if(a1==FALSE)
	 {
	   print(31,3,"<Import>");
	   scr.current=LGREEN;
	   win_cls(help);
	   win_print(help,3,2,"Import modeling parameter file for GPR forward modeling.");
	 }
	 win_redraw_all();
    }
  }
  if(key==ESC) a1=ESC;
  if(key==F10)
  {
    check_quit();  /* If user wants to exit */
    a1=import_or_create();
  }
  return(a1);
}

/*************************************************************************/
int import_fname(int option) /* If import file, ask user name of the file */
{
  char tmp[80];
  int a2;
  strcpy(tmp,"\0");

  if(option==0)
  {
    scr.current=LGREEN;
    win_cls(help);
    win_print(help,3,1,"Enter name of a previously saved calibrated transmission wavelet file.");
    win_print(help,3,2,"Do not include file extension.");
    win_print(help,3,3,"File should be saved with '.TRN' extension.");
  }
  if(option==1)
  {
    scr.current=LGREEN;
    win_cls(help);
    win_print(help,3,1,"Enter name of a previously saved data file.");
    win_print(help,3,2,"Do not include file extension.");
    win_print(help,3,3,"File should be saved with '.MOD' extension.");
  }
  key_win(2);
  win_pop_top(work);
  scr.current=C_MAIN;
  if(option==0) {print(2,4,"Enter transmission file ==>");}
  if(option==1) {print(5,4,"Enter file to import ==>");}
  a2=line_editor(32,4,78,C_MAIN,tmp,TRUE,TRUE);
  if(a2!=ESC && a2!=F10 && option==0) {strcpy(ftran,tmp);
								strcat(ftran,".TRN");}
  if(a2!=ESC && a2!=F10 && option==1) {strcpy(fproc,tmp);
								strcat(fproc,".MOD");}
  if(a2==F10)
  {
    check_quit(); /* If user wants to exit */
    a2=import_fname(option);
  }
  return(a2);
}

/*************************************************************************/
int desire_change_param()      /* Ask if parameter wants to be changed */
{
  int key=0,a3=FALSE,i;

  cursor(NO_CURSOR);

  do
  {
    if(change==0) change=win_make(6,18,70,1,NO_FRAME,"",BLACK,C_MAIN);
    else {win_pop_top(change); ceol(1,1);}
    cursor(NO_CURSOR);
    scr.current=C_MAIN;
    print(1,1,"Change a parameter ==>    No      Yes ");
    scr.current=scr.inverse;
    print(26,1,"<No>");
    scr.current=BOLD;
    print(35,1,"Y");
    win_cls(help);
    scr.current=LGREEN;
    win_print(help,3,1,"Proceed with GPR Forward modeling using the above values.");
    win_print(help,3,3,"Press <Esc> to return to <Import>/<Create> option.");
    key_win(1);

    while((key=getch())!=RETURN && key!=ESC)
    {
	 if(key==0 || key==0xE0) key=getch();
	 if(key==F10 || key==ALT_S || key==ALT_P || key==ALT_V || key==ALT_O \
		 || key==ALT_R || key==ALT_F || key==ALT_T) break;
	 if(key==78 || key==110) {a3=FALSE; key=RETURN; break;}
	 if(key==89 || key==121) 
	 {
	   a3=TRUE;
	   key=RETURN;
	   scr.current=C_MAIN;
	   print(26,1," No ");
	   scr.current=scr.inverse;
	   print(34,1,"<Yes>");
	   scr.current=BOLD;
	   print(27,1,"N");
	   break;
	 }
	 if(key==LEFT || key==RIGHT)
	 {
	   if(key==RIGHT) if(a3==FALSE) a3=TRUE;
	   if(key==LEFT ) if(a3==TRUE) a3=FALSE;
	   scr.current=C_MAIN;
	   print(1,1,"Change a parameter ==>    No      Yes ");
	   scr.current=BOLD;
	   print(35,1,"Y");
	   print(27,1,"N");
	   scr.current=scr.inverse;
	   if(a3==TRUE)
	   {
		print(34,1,"<Yes>");
		win_cls(help);
		scr.current=LGREEN;
		win_print(help,3,1,"Change a parameter listed above.");
		win_print(help,3,3,"Press <Esc> to return to <Import>/<Create> option.");
	   }
	   if(a3==FALSE)
	   {
		print(26,1,"<No>");
		win_cls(help);
		scr.current=LGREEN;
		win_print(help,3,1,"Proceed with GPR Forward modeling using the above values.");
		win_print(help,3,3,"Press <Esc> to return to <Import>/<Create> option.");
	   }
	 }
	 win_redraw_all();
    }
    if(key==ALT_S) save_file(0);
    if(key==ALT_P) plot_info();
    if(key==ALT_V) vary_toggle();
    if(key==ALT_O) options();
    if(key==ALT_R) resample(1,dtdata,t_sample,sample,g_mode); 
    if(key==ALT_F) 
    {
	 import_field(1);
	 for(i=1;i<=N;i++)  dtdata[i]=dtdata_in[i]; 
	 resample(0,dtdata,t_sample,sample/c_ratio,g_mode); 
    }
    if(key==ALT_T) 
    {
	 import_tran(1);
	 resample(0,dtdata,t_sample,sample,g_mode); 
    }                            /* c_ratio set to 1.0 */
    if(key==ESC) {a3=ESC; break;}
    if(key==F10) break;
    if(key==RETURN) break;
  } while(TRUE);

  if(key==F10)
  {
    check_quit(); /* if user wants to quit */
    a3=desire_change_param();
  }
  return(a3);
}

/*************************************************************************/
int which_param()     /* If parameter wants to be changed, enter which parameter */
{
  char tmp[100];
  int a4,key,q=1,i;
  double t;

  do
  {
    switch(q)
    {
	 case 1:
	   a4=1;
	   scr.current=C_MAIN;
	   win_pop_top(change);
	   ceol(1,1);
	   cursor(NO_CURSOR);
	   print(1,1,"Change parameter ==>   Layer   t   First   Last   Offset ");
	   scr.current=BOLD;
	   print(24,1,"L");
	   print(33,1,"t"); 
	   print(37,1,"F");
	   print(46,1,"a"); 
	   print(52,1,"O");
	   scr.current=scr.inverse;
	   print(23,1,"<Layer>");
	   win_cls(help);
	   scr.current=LGREEN;
	   win_print(help,3,1,"Change a layer parameter listed above.");
	   win_print(help,3,2,"Also, option to insert or delete a layer.");
	   key_win(1);

	   while((key=getch())!=RETURN && key!=ESC)
	   {
		if(key==0 || key==0xE0) key=getch();
		if(key==F10) break;
		if(key==76 || key==108) {a4=1; break;}
		if(key==84 || key==116) {a4=2; break;}
		if(key==70 || key==102) {a4=3; break;}
		if(key==65 || key==97) {a4=4; break;}
		if(key==79 || key==111) {a4=5; break;}
		if(key==LEFT || key==RIGHT)
		{
		  if(key==LEFT ) if(a4>1) a4--;
		  if(key==RIGHT) if(a4<5) a4++;
		  scr.current=C_MAIN;
		  print(1,1,"Change parameter ==>   Layer   t   First   Last   Offset ");
		  scr.current=BOLD; 
		  print(24,1,"L");
		  print(33,1,"t");
		  print(37,1,"F");
		  print(46,1,"a");
		  print(52,1,"O");
		  scr.current=scr.inverse;
		  if(a4==1)
		  {
		    print(23,1,"<Layer>");
		    win_cls(help);
		    scr.current=LGREEN;
		    win_print(help,3,1,"Change a layer parameter listed above.");
		    win_print(help,3,2,"Also, option to insert or delete a layer.");
		  }
		  if(a4==2)
		  {
		    print(31,1,"<t>");
		    win_cls(help);
		    scr.current=LGREEN;
		    win_print(help,3,2,"Change forward modeling sample rate.");
		  }
		  if(a4==3)
		  {
		    print(36,1,"<First>");
		    win_cls(help);
		    scr.current=LGREEN;
		    win_print(help,3,1,"Change value of the first data point to plot.");
		    win_print(help,3,2,"This value should be zero in the final forward model,");
		    win_print(help,3,3,"since forward models are calculated from point zero.");
		  }
		  if(a4==4)
		  {
		    print(44,1,"<Last>");
		    win_cls(help);
		    scr.current=LGREEN;
		    win_print(help,3,2,"Change value of the last data point to plot.");
		  }
		  if(a4==5)
		  {
		    print(51,1,"<Offset>");
		    win_cls(help);
		    scr.current=LGREEN;
		    win_print(help,3,1,"Change value of offset between field data and forward model.");
		    win_print(help,3,2,"To find correct offset, set <first> to zero and adjust <offset>");
		    win_print(help,3,3,"so that the field scan is in the correct time position.");
		  }
		  win_redraw_all();
		}
	   }
	   if(a4!=1) q++;
	   if(key==ESC || a4==1) q=(-1);
	   break;
	 case 2:
	   scr.current=scr.inverse;
	   if(a4==2)
	   {
		sprintf(tmp,"%8.3f ",sample);
		win_print(list,66,1,tmp);
		win_cls(help);
		scr.current=LGREEN;
		win_print(help,3,2,"Enter new forward modeling sample rate [ns] [0.01 - 5.0].");
	   }
	   if(a4==3)
	   {
		sprintf(tmp,"%8.3f ",first*sample);
		win_print(list,66,2,tmp);
		sprintf(tmp,"Enter new first data point to plot [0.0 to %8.4f] [ns].",sample*511.);
		win_cls(help);
		scr.current=LGREEN;
		win_print(help,3,2,tmp);
	   }
	   if(a4==4)
	   {
		sprintf(tmp,"%8.3f ",last*sample);
		win_print(list,66,3,tmp);
		sprintf(tmp,"Enter new last data point to plot [0.0 to %8.4f] [ns].",sample*511.);
		win_cls(help);
		scr.current=LGREEN;
		win_print(help,3,2,tmp);
	   }
	   if(a4==5)
	   {
		sprintf(tmp,"%8.3f ",offset*sample);
		win_print(list,66,4,tmp);
		win_cls(help);
		scr.current=LGREEN;
		win_print(help,3,2,"Enter new offset between field and model data (in nanoseconds).");
	   }
	   key_win(2);
	   ceol(1,1);
	   scr.current=C_MAIN;
	   print(2,1,"Enter value ==>");
	   strcpy(tmp,"\0");
	   key=line_editor(18,1,78,C_MAIN,tmp,FALSE,TRUE);
	   if(key==ESC)
	   {
		win_delete(list);
		list_file();
		q=1;
		break;
	   }
	   if(key==F10) break;
	   if(tmp[0]==NULL) break;
	   t=atof(tmp);
	   if(a4==2)
	   {
		if(t==0.0) t=f_sample;
		if(t<0.01 || t>5.0) {printf("\a"); q=1;}
		else 
		{
		  for(i=1;i<=N;i++)  dtdata[i]=dtdata_in[i]; 
		  resample(0,dtdata,t_sample,t/c_ratio,g_mode); 
		  for(i=1;i<=N;i++)  dfield[i]=dfield_in[i]; 
		  resample(0,dfield,f_sample,t,g_mode); 
		  offset*=(sample/t); sample=t; 
		}
	   }
	   if(a4==3)
	   {
		if(t<0.0 || t>sample*511.0) {printf("\a"); q=1;}
		else first=t/sample;
	   }
	   if(a4==4)
	   {
		if(t<0.0 || t>sample*511.0) {printf("\a"); q=1;}
		else last=t/sample;
	   }
	   if(a4==5)
	   {
		if(t<(-511.0) || t>511.0) {printf("\a"); q=1;}
		else offset=t/sample;
	   }
	   if(sample<0.01 && tmp[0]==NULL) {printf("\a"); q=1;}
	   q++;
	   break;
    }
    if(key==F10)
    {
	 check_quit(); /* if user wants to quit */
	 a4=which_param();
	 q=(-1);
    }
    if(tmp[0]==NULL) break;
  } while(q!=(-1) && q!=3);

  if(key==ESC) a4=ESC;
  return(a4);
}

/*************************************************************************/
int which_layer() /* If layer parameter wants to be changed, enter which layer */
{
  char lab[100],lab2[100],tmp1[50],tmp2[50],tmp3[5];
  int a5,i,l,key,m,q=0,flag=0;

  do
  {
    a5=1;
    l=lnum+1;
    win_pop_top(change);
    ceol(1,1);
    cursor(NO_CURSOR);
    strcpy(lab,"Layer number ==> ");
    strcpy(lab2,"\0");
    for(i=1;i<=lnum;i++)
    {
	 sprintf(tmp1," %d  \0",i);
	 strcat(lab,tmp1);
	 strcat(lab2,tmp1);
    }
    if(flag==0 && lnum>2 && lnum<9) /* change previously existing layer parameter */
    {
	 strcat(lab," Ins   Del ");
	 win_cls(help);
	 scr.current=LGREEN;
	 win_print(help,3,2,"Alter parameter in layer 1.");
	 m=lnum+1;
    }
    if(flag==0 && lnum<=2) /* change previously existing layer parameter */
    {
	 strcat(lab," Ins ");
	 win_cls(help);
	 scr.current=LGREEN;
	 win_print(help,3,2,"Alter parameter in layer 1.");
	 m=lnum;
    }
    if(flag==0 && lnum==9) /* change previously existing layer parameter */
    {
	 strcat(lab,"Del ");
	 win_cls(help);
	 scr.current=LGREEN;
	 win_print(help,3,2,"Alter parameter in layer 1.");
	 m=lnum;
    }
    if(flag==1) /* Insert a layer */
    {
	 sprintf(tmp1," %d  \0",lnum+1);
	 strcat(lab,tmp1);
	 win_cls(help);
	 scr.current=LGREEN;
	 win_print(help,3,2,"Insert layer 1.");
	 m=lnum;
	 l=lnum+2;
    }
    if(flag==2) /* Delete a layer */
    {
	 win_cls(help);
	 scr.current=LGREEN;
	 win_print(help,3,2,"Delete layer 1.");
	 m=lnum-1;
	 l=lnum+1;
    }
    ceol(1,1);
    scr.current=C_MAIN;
    print(2,1,lab);
    scr.current=BOLD;
    print(19,1,lab2);
    if(flag==0 && lnum<=2) {print(28,1,"I");}
    else if(flag==0 && lnum==9) {print(55,1,"D");}
    else if(flag==0) 
    {
	 print(20+(lnum)*4,1,"I");
	 print(22+((lnum+1)*4),1,"D");
    }
    if(flag==1) 
    {
	 sprintf(tmp3,"%d",lnum+1);
	 print(20+(lnum)*4,1,tmp3);
    }
    scr.current=scr.inverse;
    print(19,1,"<1>");
    key_win(1);

    while((key=getch())!=RETURN && key!=ESC)
    {
	 if(key==0 || key==0xE0) key=getch();
	 if(key==F10) break;
	 if(key>=49 && key<=48+lnum) {a5=key-48; break;}
	 if(flag==1 && key==48+lnum+1) {a5=key-48; break;}
	 if(flag==0 && lnum>2 && lnum<9 && (key==73 || key==105)) {a5=l; break;}
	 if(flag==0 && lnum>2 && lnum<9 && (key==68 || key==100)) {a5=lnum+2; break;}
	 if(flag==0 && lnum<=2 && (key==73 || key==105)) {a5=l; break;}
	 if(flag==0 && lnum==9 && (key==68 || key==100)) {a5=lnum+1; break;}
	 if(key==LEFT || key==RIGHT)
	 {
	   if(key==LEFT ) if(a5>1) a5--;
	   if(key==RIGHT) if(a5<=m) a5++;
	   scr.current=C_MAIN;
	   ceol(1,1);
	   print(2,1,lab);
	   if(a5==l && lnum<9)
	   {
		strcpy(tmp1,"Insert a layer into model data.");
		strcpy(tmp2,"<Ins>");
	   }
	   else if(a5==lnum+2 && lnum>2 && lnum<9)
	   {
		strcpy(tmp1,"Delete a layer from model data.");
		strcpy(tmp2,"<Del>");
	   }
	   else if(a5==lnum+1 && lnum==9)
	   {
		strcpy(tmp1,"Delete a layer from model data.");
		strcpy(tmp2,"<Del>");
	   }
	   else
	   {
		sprintf(tmp2,"<%d>",a5);
		if(flag==0) sprintf(tmp1,"Alter parameter in layer %d.",a5);
		if(flag==1) sprintf(tmp1,"Insert layer %d.",a5);
		if(flag==2) sprintf(tmp1,"Delete layer %d.",a5);
	   }
	   scr.current=BOLD; 
	   print(19,1,lab2);
	   if(flag==0 && lnum<=2) {print(28,1,"I");}
	   else if(flag==0 && lnum==9) {print(55,1,"D");}
	   else if(flag==0)
	   {
		print(20+(lnum)*4,1,"I");
		print(22+((lnum+1)*4),1,"D");
	   }
	   if(flag==1) 
	   {
		sprintf(tmp3,"%d",lnum+1);
		print(20+(lnum)*4,1,tmp3);
	   }
	   scr.current=scr.inverse;
	   if(a5!=lnum+2) {print(19+(a5-1)*4,1,tmp2);}
	   else {print(20+(a5-1)*4,1,tmp2);}
	   win_cls(help);
	   scr.current=LGREEN;
	   win_print(help,3,2,tmp1);
	 }
	 win_redraw_all();
    }
    if(key==F10) break;
    if(a5==lnum+1 && q==0 && lnum<9) flag=1;
    if(a5==lnum+2 && q==0 && lnum>2) flag=2;
    if(a5==lnum+1 && q==0 && lnum==9) flag=2;
    if(key==ESC && q==0) a5=ESC;
    if(key==ESC && q==1) {q=flag=0; a5=lnum+1;}
    if((a5==lnum+1 && flag==1) || (a5==lnum+2 && flag==2)) q++;
    if(key==ESC && q==1) a5=ESC;
  } while((a5==lnum+2 || a5==lnum+1) && q!=2);

  if(key!=ESC && key!=F10)
  {
    if(flag==1)
    {
	 if(a5==lnum+1) key=input_model(a5,1,1);
	 else key=input_model(a5,3,1);
	 if(key==ESC && a5!=lnum+1)
	 {
	   lnum--;
	   for(i=a5;i<=lnum;i++) 
	   {
		e[i].r=e[i+1].r; 
		e[i].i=e[i+1].i; 
		con[i]=con[i+1]; 
		dis[i]=dis[i+1];
	   }
	   win_delete(list);
	   list_file();
	   a5=ESC;
	 }
	 else a5=(-1);
    }
    if(flag==2)
    {
	 if(a5!=lnum) 
	 {
	   for(i=a5;i<=lnum;i++) 
	   {
		e[i].r=e[i+1].r; 
		e[i].i=e[i+1].i; 
		con[i]=con[i+1]; 
		dis[i]=dis[i+1];
	   }
	 }
	 lnum--;
	 a5=(-1);
    }
  }
  if(key==F10)
  {
    check_quit();
    a5=which_layer();
  }
  win_delete(list);
  win_cls(work);
  list_file();
  return(a5);
}

/*************************************************************************/
int input_model(int l,int flag,int p)     /* Enter layer parameters */
{
  char tmp[100],a6[80];
  int i,key,ll=l;
  double t;

  cursor(NO_CURSOR);
  if(flag==1)                    /* Insert last layer */
  {
    ll=l-1;
    lnum++;
    e[lnum].r=1.;
    e[lnum].i=con[lnum]=0.;
    dis[lnum-1]=0.01;
    win_delete(list);
    list_file();
  }
  if(flag==3)                    /* Insert any layer below bottom layer */
  {
    for(i=lnum;i>=l;i--) e[i+1].r=e[i].r, e[i+1].i=e[i].i, con[i+1]=con[i], dis[i+1]=dis[i];
    lnum++;
    e[l].r=1.;
    e[l].i=con[l]=0.;
    dis[l]=0.01;
    win_delete(list);
    list_file();
  }
  if(change==0) change=win_make(6,18,70,1,NO_FRAME,"",BLACK,C_MAIN);
  else win_pop_top(change);
  for(i=p;i<=4;i++)
  {
    if(i==4 && flag==2) break;
    scr.current=scr.inverse;
    if(i==1)
    {
	 sprintf(tmp,"%8.3f  ",e[l].r);
	 win_print(list,7,1+l,tmp);
	 sprintf(tmp,"Enter value for K' [1.0 - 100.0] of layer %d.",l);
    }
    if(i==2)
    {
	 sprintf(tmp,"%8.3f  ",e[l].i);
	 win_print(list,17,1+l,tmp);
	 sprintf(tmp,"Enter value for K''[0.0 - 100.0] of layer %d.",l);
    }
    if(i==3)
    {
	 sprintf(tmp,"%9.3f  ",con[l]*1000.);
	 win_print(list,27,1+l,tmp);
	 sprintf(tmp,"Enter value for  [mS/m] [0.0 - 10000.0] of layer %d.",l);
    }
    if(i==4)
    {
	 sprintf(tmp,"%8.3f  ",dis[ll]);
	 win_print(list,41,1+ll,tmp);
	 sprintf(tmp,"Enter value for the thickness [m] [0.01 - 500.0] of layer %d.",ll);
    }
    win_cls(help);
    scr.current=LGREEN;
    win_print(help,3,2,tmp);
    win_pop_top(change);
    ceol(1,1);
    key_win(2);
    scr.current=C_MAIN;
    print(2,1,"Enter value ==>");
    strcpy(a6,"\0");
    key=line_editor(18,1,78,C_MAIN,a6,FALSE,TRUE);
    if(key!=ESC && key!=F10 && a6[0]!=NULL)
    {
	 t=atof(a6);
	 if(i==1)
	 {
	   if(t<1. || t>100.) {printf("\a"); i--;}
	   else e[l].r=t;
	 }
	 if(i==2)
	 {
	   if(t<0. || t>100.) {printf("\a"); i--;}
	   else e[l].i=t;
	 }
	 if(i==3)
	 {
	   if(t<0. || t>10000.) {printf("\a"); i--;}
	   else con[l]=t/1000.;
	 }
	 if(i==4)
	 {
	   if(t<0.01 || t>500.) {printf("\a"); i--;}
	   else dis[ll]=t;
	 }
    }
    if(e[l].r<1. && a6[0]==NULL) {printf("\a"); i--;}
    win_delete(list);
    list_file();
    if(key==F10)
    {
	 check_quit();              /* if user wants to quit */
	 i--;
    }
    if(key==ESC)
    {
	 win_delete(list);
	 list_file();
	 i-=2;
    }
    if(i<0) break;
  }
  if(i<0 && flag==1)             /* If insert last layer and ESC, delete last layer */
  {
	key=ESC;
	lnum--;
	win_delete(list);
	list_file();
  } 
  if(i==5 && flag==1) key=1;
  return(key);
}

/*************************************************************************/
int which_layer_param(int a5)     /* If change one layer parameter, choose which one */
{
  char tmp[100];
  int key,k=4,a7=1;

  win_pop_top(change);
  cursor(NO_CURSOR);
  scr.current=C_MAIN;
  if(a5==lnum) 
  {
    print(2,1,"Layer parameter ==>   Real [K']   Imag [K'']   Con [] "); 
    k=3;
  }
  else {print(2,1,"Layer parameter ==>   Real [K']   Imag [K'']   Con []   Thickness ");}
  scr.current=BOLD; 
  print(24,1,"R");
  print(36,1,"I"); 
  print(49,1,"C");
  if(a5!=lnum) {print(59,1,"T");}
  scr.current=scr.inverse;
  print(23,1,"<Real [K']>");
  sprintf(tmp,"Alter real part of the relative dielectric permittivity of layer %d.",a5);
  win_cls(help);
  scr.current=LGREEN;
  win_print(help,3,2,tmp);
  key_win(1);

  while((key=getch())!=RETURN && key!=ESC)
  {
    if(key==0 || key==0xE0) key=getch();
    if(key==F10) break;
    if(key==82 || key==114) {a7=1; break;}
    if(key==73 || key==105) {a7=2; break;}
    if(key==67 || key==99) {a7=3; break;}
    if(k!=3 && (key==84 || key==116)) {a7=4; break;}
    if(key==LEFT || key==RIGHT)
    {
	 if(key==LEFT ) if(a7>1) a7--;
	 if(key==RIGHT) if(a7<k) a7++;
	 scr.current=C_MAIN;
	 if(a5==lnum) {print(2,1,"Layer parameter ==>   Real [K']   Imag [K'']   Con [] ");}
	 else {print(2,1,"Layer parameter ==>   Real [K']   Imag [K'']   Con []   Thickness ");}
	 scr.current=BOLD; 
	 print(24,1,"R");
	 print(36,1,"I"); 
	 print(49,1,"C");
	 if(a5!=lnum) {print(59,1,"T");}
	 scr.current=scr.inverse;
	 if(a7==1)
	 {
	   print(23,1,"<Real [K']>");
	   sprintf(tmp,"Alter real part of the relative dielectric permittivity of layer %d.",a5);
	 }
	 if(a7==2)
	 {
	   print(35,1,"<Imag [K'']>");
	   sprintf(tmp,"Alter imaginary part of the relative dielectric permittivity of layer %d.",a5);
	 }
	 if(a7==3)
	 {
	   print(48,1,"<Con []>");
	   sprintf(tmp,"Alter conductivity [mS/m] of layer %d.",a5);
	 }
	 if(a7==4)
	 {
	   print(58,1,"<Thickness>");
	   sprintf(tmp,"Alter thickness [m] of layer %d.",a5);
	 }
	 win_cls(help);
	 scr.current=LGREEN;
	 win_print(help,3,2,tmp);
    }
    win_redraw_all();
  }

  if(key==ESC) a7=ESC;
  if(key==F10)
  {
    check_quit();                /* if user wants to quit */
    a7=which_layer_param(a5);
  }
  return(a7);
}

/*************************************************************************/
int new_layer_param(int a5,int a7)   /* If change one layer parameter, enter new value */
{
  char tmp[100],a8[80];
  int i,key;
  double t;

  if(change==0) change=win_make(6,18,70,1,NO_FRAME,"",BLACK,C_MAIN);
  else win_pop_top(change);

  do
  {
    scr.current=scr.inverse;
    if(a7==1)
    {
	 sprintf(tmp,"%8.3f  ",e[a5].r);
	 win_print(list,7,1+a5,tmp);
	 sprintf(tmp,"Enter value for K' [1.0 - 100.0] of layer %d.",a5);
    }
    if(a7==2)
    {
	 sprintf(tmp,"%8.3f  ",e[a5].i);
	 win_print(list,17,1+a5,tmp);
	 sprintf(tmp,"Enter value for K''[0.0 - 100.0] of layer %d.",a5);
    }
    if(a7==3)
    {
	 sprintf(tmp,"%9.3f  ",con[a5]*1000.);
	 win_print(list,27,1+a5,tmp);
	 sprintf(tmp,"Enter value for  [mS/m] [0.0 - 10000.0] of layer %d.",a5);
    }
    if(a7==4)
    {
	 sprintf(tmp,"%8.3f  ",dis[a5]);
	 win_print(list,41,1+a5,tmp);
	 sprintf(tmp,"Enter value for the thickness [m] [0.01 - 500.0] of layer %d.",a5);
    }
    win_cls(help);
    scr.current=LGREEN;
    win_print(help,3,2,tmp);
    ceol(1,1);
    key_win(2);
    scr.current=C_MAIN;
    print(2,1,"Enter new value ==>");
    strcpy(a8,"\0");
    key=line_editor(22,1,78,C_MAIN,a8,FALSE,TRUE);
    if(key!=ESC && key!=F10 && a8[0]!=NULL)
    {
	 t=atof(a8);
	 if(a7==1)
	 {
	   if(t<1. || t>100.) {printf("\a"); continue;}
	   else e[a5].r=t;
	 }
	 if(a7==2)
	 {
	   if(t<0. || t>100.) {printf("\a"); continue;}
	   else e[a5].i=t;
	 }
	 if(a7==3)
	 {
	   if(t<0. || t>10000.) {printf("\a"); continue;}
	   else con[a5]=t/1000.;
	 }
	 if(a7==4)
	 {
	   if(t<0.01 || t>500.) {printf("\a"); continue;}
	   else dis[a5]=t;
	 }
    }
    if(e[a5].r<1. && a8[0]==NULL) {printf("\a"); continue;}
    if(list) win_delete(list);
    list_file();
    if(key!=F10 && key!=ESC) key=1;
    break;
  } while(key!=ESC && key!=F10);

  if(key==F10)
  {
    check_quit();
    key=new_layer_param(a5,a7);
    key=1;
  }
  return(key);
}

/*************************************************************************/
int input_plt_title()      /* If create file, enter plot title */
{
  int a9;

  cursor(NO_CURSOR);
  win_cls(help);
  scr.current=LGREEN;
  win_print(help,3,2,"Enter plot title.");
  key_win(2);
  scr.current=C_MAIN;
  print(4,6,"Plot title (<45 char) ==> ");
  strcpy(title,"\0");
  a9=line_editor(32,6,76,C_MAIN,title,FALSE,FALSE);
  if(a9==F10)
  {
    check_quit(); /* If user wants to quit */
    a9=input_plt_title();
  }
  return(a9);
}

/*************************************************************************/
int num_layers()     /* If create file, enter number of layers to create */
{
  char tmp[50];
  int a10,i=0;

  win_cls(help);
  scr.current=LGREEN;
  win_print(help,3,2,"Enter number of model layers [2 - 9].");
  key_win(2);

  do
  {
    if(i>0) printf("\a");
    ceol(1,5);
    scr.current=C_MAIN;
    print(3,5,"Enter number of layers ==>");
    strcpy(tmp,"\0");
    a10=line_editor(32,5,78,C_MAIN,tmp,FALSE,TRUE);
    if(a10==ESC || a10==F10) break;
    i++;
  } while((lnum=atoi(tmp))<2 || lnum>9);

  if(a10!=ESC && a10!=F10)
  {
    first=offset=0.; last=511.;
    for(i=1;i<=lnum;i++) {e[i].r=1.; e[i].i=con[i]=0.;
    dis[i]=0.01;}
  }
  if(a10==F10)
  {
    check_quit(); /* if user wants to quit */
    strcpy(tmp,"\0");
    a10=num_layers();
  }
  return(a10);
}

/*************************************************************************/
int create_model()     /* If create file, enter layer parameters */
{
  char tmp[50];
  int i,pp=1,a11,flag=0,k;
  double t;
  ratio=1.;

  win_cls(work);
  win_redraw_all();
  if(list) win_delete(list);
  list_file();

  do
  {
    for(i=1;i<=lnum;i++)
    {
	 if(i==lnum) flag=2;
	 a11=input_model(i,flag,pp);
	 pp=1;
	 if(a11==ESC)
	 {
	   win_delete(list);
	   list_file();
	   i-=2;
	   pp=4;
	   if(flag==2) flag=0;
	 }
	 if(a11==F10)
	 {
	   check_quit();
	   i--;
	 }
	 if(i<0) break;
    }
    if(i<0) break;
    if(change==0) change=win_make(6,18,70,1,NO_FRAME,"",BLACK,C_MAIN);
    else win_pop_top(change);
    for(i=1;i<=4;i++)
    {
	 scr.current=scr.inverse;
	 if(i==1)
	 {
	   sprintf(tmp,"%8.3f ",sample);
	   win_print(list,66,1,tmp);
	   win_cls(help);
	   scr.current=LGREEN;
	   win_print(help,3,2,"Enter forward modeling sample rate [ns] [0.01 - 5.0].");
	 }
	 if(i==2)
	 {
	   sprintf(tmp,"%8.3f ",first*sample);
	   win_print(list,66,2,tmp);
	   sprintf(tmp,"Enter first data point to plot [0.0 to %8.4f] [ns]",sample*511.);
	   win_cls(help);
	   scr.current=LGREEN;
	   win_print(help,3,1,tmp);
	   win_print(help,3,2,"This value should be zero in the final forward model,");
	   win_print(help,3,3,"since forward models are calculated from point zero.");
	 }
	 if(i==3)
	 {
	   sprintf(tmp,"%8.3f ",last*sample);
	   win_print(list,66,3,tmp);
	   sprintf(tmp,"Enter last data point to plot [0.0 to %8.4f] [ns]",sample*511.);
	   win_cls(help);
	   scr.current=LGREEN;
	   win_print(help,3,2,tmp);
	 }
	 if(i==4)
	 {
	   sprintf(tmp,"%8.3f ",offset*sample);
	   win_print(list,66,4,tmp);
	   win_cls(help);
	   scr.current=LGREEN;
	   win_print(help,3,1,"Enter offset between field data and forward model (in nanoseconds).");
	   win_print(help,3,2,"To find correct offset, set <first> to zero and adjust <offset>");
	   win_print(help,3,3,"so that the field scan is in the correct time position.");
	 }
	 win_pop_top(change);
	 ceol(1,1);
	 win_redraw_all();
	 scr.current=C_MAIN;
	 print(2,1,"Enter value ==>");
	 strcpy(tmp,"\0");
	 a11=line_editor(18,1,78,C_MAIN,tmp,FALSE,TRUE);
	 if(tmp[0]!=NULL) t=atof(tmp);
	 if(a11==ESC)
	 {
	   win_delete(list);
	   list_file();
	   i-=2;
	   if(i<0) i=0;
	   continue;
	 }
	 if(a11==F10)
	 {
	   check_quit();
	   i--;
	   continue;
	 }
	 if(i==1 && tmp[0]!=NULL)
	 {
	   if(t==0.0) t=f_sample;
	   if(t<0.01 || t>5.0) {printf("\a"); i--;}
	   else 
	   {
		for (k=1;k<=N;k++)  dtdata[k]=dtdata_in[k]; 
		resample(0,dtdata,t_sample,t/c_ratio,g_mode); 
		for (k=1;k<=N;k++)  dfield[k]=dfield_in[k];
		resample(0,dfield,f_sample,t,g_mode); 
		offset*=(sample/t); sample=t; 
	   }
	 }
	 if(i==2 && tmp[0]!=NULL)
	 {
	   if(t<0.0 || t>sample*511.0) {printf("\a"); i--;}
	   else first=t/sample;
	 }
	 if(i==3 && tmp[0]!=NULL)
	 {
	   if(t<0.0 || t>sample*511.0) {printf("\a"); i--;}
	   else last=t/sample;
	 }
	 if(i==4 && tmp[0]!=NULL)
	 {
	   if(t<(-511.0) || t>511.0) {printf("\a"); i--;}
	   else offset=t/sample;
	 }
	 if(sample<0.01 && tmp[0]==NULL) {printf("\a"); i--;}
	 if(list) win_delete(list);
	 list_file();
    }
    win_redraw_all();
    if(i==5) break;
  } while(TRUE);

  return(a11);
}

/*************************************************************************/
int desire_raw_data()     /* Ask if raw field data should be imported */
{
  int key=0,a12=FALSE;

  win_pop_top(work);
  cursor(NO_CURSOR);
  win_cls(help);
  scr.current=LGREEN;
  win_print(help,3,1,"Do not import a raw scan data for range-gain removal.");
  win_print(help,3,2,"Either forward models will not be overlayed with a field");
  win_print(help,3,3,"scan or a previously range-corrected field scan may be imported.");
  key_win(1);
  scr.current=C_MAIN;
  print(4,7,"Import raw field data ==>   No      Yes ");
  scr.current=BOLD; 
  print(40,7,"Y");
  scr.current=scr.inverse; 
  print(31,7,"<No>");

  while((key=getch())!=RETURN && key!=ESC)
  {
    if(key==0 || key==0xE0) key=getch();
    if(key==F10) break;
    if(key==78 || key==110) 
    {
	 a12=FALSE; 
	 key=RETURN;
	 scr.current=C_MAIN; 
	 print(40,7," Yes ");
	 scr.current=scr.inverse; 
	 print(31,7,"<No>");
	 scr.current=BOLD; 
	 print(40,7,"Y"); 
	 break;
    }
    if(key==89 || key==121) 
    {
	 a12=TRUE; 
	 key=RETURN;
	 scr.current=C_MAIN; 
	 print(31,7," No ");
	 scr.current=scr.inverse; 
	 print(39,7,"<Yes>");
	 scr.current=BOLD; 
	 print(32,7,"N"); 
	 break;
    }
    if(key==LEFT || key==RIGHT)
    {
	 if(key==RIGHT) if(a12==FALSE) a12=TRUE;
	 if(key==LEFT ) if(a12==TRUE) a12=FALSE;
	 scr.current=C_MAIN;
	 print(4,7,"Import raw field data ==>   No      Yes ");
	 scr.current=BOLD; 
	 print(40,7,"Y");
	 print(32,7,"N");
	 scr.current=scr.inverse;
	 if(a12==TRUE)
	 {
	   print(39,7,"<Yes>");
	   win_cls(help);
	   scr.current=LGREEN;
	   win_print(help,3,1,"Import a raw field scan for range-gain removal.");
	   win_print(help,3,2,"Raw field file should contain data stored as 512 bytes, and");
	   win_print(help,3,3,"should be saved with '.RAW' extension.");
	   win_redraw_all();
	 }
	 if(a12==FALSE)
	 {
	   print(31,7,"<No>");
	   win_cls(help);
	   scr.current=LGREEN;
	   win_print(help,3,1,"Do not import a raw field scan for range-gain removal.");
	   win_print(help,3,2,"Either forward models will not be overlayed with a field");
	   win_print(help,3,3,"scan or a previously range-gain corrected field scan may be imported.");
	   win_redraw_all();
	 }
    }
  }
  if(key==ESC) a12=ESC;
  if(key==F10)
  {
    check_quit();                /* if user wants to quit */
    a12=desire_raw_data();
  }
  return(a12);
}

/*************************************************************************/
int import_raw_rg_tc()     /* Import raw field and range gain files */
{
  char tmp[80],root[80];
  int i,k,a13;
  FILE *raw_file,*gain_file,*time_file;

  win_pop_top(work);
  key_win(2);
  for(i=1;i<=3;i++)
  {
    if(i==1)
    {
	 win_cls(help);
	 scr.current=LGREEN;
	 win_print(help,3,1,"Enter name of raw field file.");
	 win_print(help,3,2,"File should be stored as 512 char bytes and given '.RAW' extension.");
	 strcpy(tmp,"\0");

	 do
	 {
	   scr.current=C_MAIN;
	   print(3,8,"Raw Field Data Filename ==>");
	   win_redraw_all();
	   a13=line_editor(32,8,78,C_MAIN,tmp,TRUE,TRUE);
	   if(a13==ESC) return(ESC);
	   if(a13==F10) break;
	   strcpy(root,tmp); strcpy(fraw,tmp); strcat(fraw,".RAW");
	   if((raw_file=fopen(fraw,"rb"))==NULL) file_not_good(5); /* import file cannot be found */
	   else break;
	 } while(TRUE);

	 if(a13==F10) break;
	 fread(craw,1,N,raw_file); fclose(raw_file);
	 k=strlen(fraw);
	 fraw[k-4]='\0';
    }
    if(i==2)
    {
	 win_cls(help);
	 scr.current=LGREEN;
	 win_print(help,3,1,"Enter name of range gain file.");
	 win_print(help,3,2,"File should be stored as 512 char bytes and given '.RG' extension.");
	 strcpy(tmp,"\0");

	 do
	 {
	   scr.current=C_MAIN;
	   print(7,9,"Range Gain Filename ==>");
	   win_redraw_all();
	   a13=line_editor(32,9,78,C_MAIN,tmp,TRUE,TRUE);
	   if(a13==ESC || a13==F10) break;
	   if(strlen(tmp)>1) {strcpy(fgain,tmp);} else {strcpy(fgain,root);}
	   strcat(fgain,".RG");
	   if((gain_file=fopen(fgain,"rb"))==NULL) file_not_good(6); 
	   else {fclose(gain_file); break;}
	 } while(TRUE);

	 if(a13==F10) break;
	 if(a13==ESC) i-=2;
    }
    if(i==3)
    {
	 win_cls(help);
	 scr.current=LGREEN;
	 win_print(help,3,1,"Enter name of time calibration file.");
	 win_print(help,3,2,"File should be stored as 512 char bytes and given '.TC' extension.");
	 strcpy(tmp,"\0");

	 do
	 {
	   scr.current=C_MAIN;
	   print(1,10,"Time Calibration Filename ==>");
	   win_redraw_all();
	   a13=line_editor(32,10,78,C_MAIN,tmp,TRUE,TRUE);
	   if(a13==ESC) return(ESC);
	   if(a13==F10) break;
	   if(strlen(tmp)>1) {strcpy(ftime,tmp);} else {strcpy(ftime,root);}
	   strcat(ftime,".TC");
	   if((time_file=fopen(ftime,"rb"))==NULL) file_not_good(7); 
	   else break;
	 } while(TRUE);

	 if(a13==ESC) i-=2;
    }
  }
  if(a13==F10)
  {
    check_quit();                /* If user wants to exit */
    a13=import_raw_rg_tc();
  }
  win_cls(help);
  return(a13);
}

/*************************************************************************/
void save_file(int option)       /* Prompt to save file */
{
  char tmp[80],key=97;
  int sav=TRUE,over=FALSE,i,j,k,ii=0;
  FILE *s_file;

  if(option)
  {
    key_win(1);
    win_cls(help);
    scr.current=LGREEN;
    win_print(help,3,2,"Save current modeling parameters to a file.");
    save=win_make(22,9,37,1,DBL_FRAME,"",RED,RED);
    cursor(NO_CURSOR);
    scr.current=C_MAIN;
    print(6,1,"SAVE FILE:   Yes    No ");
    scr.current=BOLD; 
    print(26,1,"N");
    scr.current=scr.inverse;
    print(18,1,"<Yes>");
    win_redraw_all();

    while((key=getch())!=RETURN && key!=ESC)
    {
	 if(key==0 || key==0xE0) key=getch();
	 if(key==F10) break;
	 if(key==78 || key==110) {sav=FALSE; break;}
	 if(key==89 || key==121) {sav=TRUE; break;}
	 if(key==LEFT || key==RIGHT)
	 {
	   if(key==LEFT ) if(sav==FALSE) sav++;
	   if(key==RIGHT) if(sav==TRUE) sav--;
	   scr.current=C_MAIN;
	   print(6,1,"SAVE FILE:   Yes    No ");
	   scr.current=BOLD; 
	   print(26,1,"N");
	   print(19,1,"Y");
	   scr.current=scr.inverse;
	   if(sav==TRUE)
	   {
		print(18,1,"<Yes>");
		win_cls(help);
		scr.current=LGREEN;
		win_print(help,3,2,"Save current modeling parameters to a file.");
	   }
	   if(sav==FALSE)
	   {
		print(25,1,"<No>");
		win_cls(help);
		scr.current=LGREEN;
		win_print(help,3,2,"Do not save current changes to a file.");
	   }
	   win_redraw_all();
	 }
	 win_redraw_all();
    }

    win_delete(save);
  }
  if(key!=ESC && key!=F10 && (sav==TRUE || option==0))
  {
    win_cls(help);
    scr.current=LGREEN;
    win_print(help,3,1,"Enter name of file to save.");
    win_print(help,3,2,"Do not include file extension.");
    win_print(help,3,3,"File will be saved with '.MOD' extension.");
    key_win(2);

    do
    {
	 save=win_make(22,9,37,2,STD_FRAME,"",YELLOW,C_MAIN);
	 cursor(NO_CURSOR);
	 scr.current=C_MAIN;
	 print(13,1," SAVE ");
	 print(2,2,"Enter File ==>");
	 gotoxy(19,2);
	 if((strcmp(fproc,"UNTITLED"))!=0)
	 {
	   strcpy(tmp,fproc);
	   k=strlen(tmp);
	   tmp[k-4]='\0';
	 }
	 else strcpy(tmp,"\0");
	 key=line_editor(19,2,58,C_MAIN,tmp,TRUE,TRUE);
	 if(key==ESC || key==F10) {win_delete(save); break;}
	 strcat(tmp,".MOD");
	 if((s_file=fopen(tmp,"rt"))!=NULL) /* Check to see if file exists */
	 {
	   fclose(s_file);
	   win_delete(save);
	   win_delete(change);
	   change=0;
	   key_win(1);
	   overw=win_make(21,9,36,1,STD_FRAME,"",WHITE,RED);
	   scr.current=C_MAIN;
	   print(1,1,"OVERWRITE FILE:   No    Yes ");
	   scr.current=BOLD; 
	   print(25,1,"Y");
	   scr.current=scr.inverse;
	   print(18,1,"<No>");
	   win_cls(help);
	   scr.current=LGREEN;
	   win_print(help,3,2,"Do not overwrite current existing file.");
	   win_redraw_all();
 
	   while((key=getch())!=RETURN && key!=ESC)
	   {
		if(key==0 || key==0xE0) key=getch();
		if(key==F10) break;
		if(key==78 || key==110) {over=FALSE; break;}
		if(key==89 || key==121) {over=TRUE; break;}
		if(key==LEFT || key==RIGHT)
		{
		  if(key==LEFT ) if(over==TRUE) over--;
		  if(key==RIGHT) if(over==FALSE) over++;
		  scr.current=C_MAIN;
		  print(1,1,"OVERWRITE FILE:   No    Yes ");
		  scr.current=BOLD; 
		  print(19,1,"N");
		  print(25,1,"Y");
		  scr.current=scr.inverse;
		  if(over==TRUE)
		  {
		    print(24,1,"<Yes>");
		    win_cls(help);
		    scr.current=LGREEN;
		    win_print(help,3,2,"Overwrite existing file.");
		  }
		  if(over==FALSE)
		  {
		    print(18,1,"<No>");
		    win_cls(help);
		    scr.current=LGREEN;
		    win_print(help,3,2,"Do not overwrite existing file.");
		  }
		}
		win_redraw_all();
	   }

	   win_delete(overw);
	   if(over==FALSE || key==ESC) continue; /* No don't overwrite file */
	   if(key==F10) break;
	 }
	 win_delete(save);

	 /** SAVE Processing file and write to parm[] for check_save() ******/
	 strcpy(fproc,tmp);
	 s_file=fopen(fproc,"wt");
	 fprintf(s_file,"Plot title ==> %s\n",title);
	 strcpy(tittmp,title);
	 fprintf(s_file,"# of layers ==> %d\n",lnum);
	 parm[1]=(double)lnum;
	 fprintf(s_file,"t ==> %f\n",sample);
	 parm[2]=sample;
	 fprintf(s_file,"time offset ==> %f\n",offset*sample);
	 parm[3]=offset;
	 fprintf(s_file,"first ==> %f\n",first*sample);
	 parm[4]=first;
	 fprintf(s_file,"last ==> %f\n",last*sample);
	 parm[5]=last;
	 fprintf(s_file,"transmission file ==> %s\n",ftran);
	 strcpy(ttmp,ftran);
	 fprintf(s_file,"field file ==> %s\n",ffield);
	 strcpy(ftmp,ffield);
	 fprintf(s_file,"coupling ratio ==> %f\n",c_ratio);
	 parm[6]=c_ratio;
	 for(i=1;i<=lnum;i++)
	 {
	   fprintf(s_file,"Layer ==> %d\n",i);
	   fprintf(s_file,"er ==> %f\n",e[i].r);
	   parm[7+ii]=e[i].r;
	   fprintf(s_file,"ei ==> %f\n",e[i].i);
	   parm[8+ii]=e[i].i;
	   fprintf(s_file,"conductivity ==> %f\n",con[i]);
	   parm[9+ii]=con[i];
	   if(i!=lnum)
	   {
		fprintf(s_file,"thickness ==> %f\n",dis[i]);
		parm[10+ii]=dis[i];
		ii+=4;
	   }
	 }
	 fclose(s_file);
	 break;
    } while(TRUE);

  }
  if(key==F10)
  {
    check_quit();                /* If user wants to quit */
    save_file(option);
  }
}

/*************************************************************************/
int check_save()   /* Check if an electrical parameter has been changed */
{
  int i,ii=0;

  if(parm[1]!=(double)lnum || parm[2]!=sample || parm[3]!=offset \
	  || parm[4]!=first || parm[5]!=last || parm[6]!=c_ratio) return(1);
  for(i=1;i<=lnum;i++)
  {
    if(e[i].r!=parm[ii+7]) return(1);
    if(e[i].i!=parm[ii+8]) return(1);
    if(con[i]!=parm[ii+9]) return(1);
    if(dis[i]!=parm[ii+10] && i!=lnum) return(1);
    ii+=4;
  }
  if(strcmp(tittmp,title)!=0) return(1);
  if(strcmp(ttmp,ftran)!=0) return(1);
  if(strcmp(ftmp,ffield)!=0) return(1);
  return(0);
}

/*************************************************************************/
void read_file(FILE *p_file) /* read previously created processing file */
{
  char lab[50];
  int i,ii=0,l;
  float real_perm,imag_perm,co,di,samp,off,fir,las,rat;

  /* Read processing file and write to parm[] for check_save() **/
  fgets(tittmp,70,p_file);
  strcpy(title,&tittmp[15]);
  l=strlen(title);
  title[l-1]='\0';
  strcpy(tittmp,title);
  fscanf(p_file,"# of layers ==> %d\r\n",&lnum);
  parm[1]=(double)lnum;
  fscanf(p_file,"t ==> %f\r\n",&samp);
  sample=parm[2]=(double)samp;
  fscanf(p_file,"time offset ==> %f\r\n",&off);
  offset=parm[3]=(double)off/sample;
  fscanf(p_file,"first ==> %f\r\n",&fir);
  first=parm[4]=(double)fir/sample;
  fscanf(p_file,"last ==> %f\r\n",&las);
  last=parm[5]=(double)las/sample;
  fscanf(p_file,"transmission file ==> %s\r\n",ftran);
  strcpy(ttmp,ftran);
  fscanf(p_file,"field file ==> %s\r\n",ffield);
  strcpy(ftmp,ffield);
  fscanf(p_file,"coupling ratio ==> %f\r\n",&rat);
  c_ratio=parm[6]=(double)rat;
  for(i=1;i<=lnum;i++)
  {
    fgets(lab,20,p_file);
    fscanf(p_file,"er ==> %f\r\n",&real_perm);
    e[i].r=parm[ii+7]=(double)real_perm;
    fscanf(p_file,"ei ==> %f\r\n",&imag_perm);
    e[i].i=parm[ii+8]=(double)imag_perm;
    fscanf(p_file,"conductivity ==> %f\r\n",&co);
    con[i]=parm[ii+9]=(double)co;
    if(i!=lnum)
    {
	 fscanf(p_file,"thickness ==> %f\r\n",&di);
	 dis[i]=parm[ii+10]=(double)di;
	 ii+=4;
    }
  }
  fclose(p_file);
}

/*************************************************************************/
void list_file()                 /* List file on screen */
{
  char lab[100],lab1[100],lab2[100],lab3[100],lab4[100];
  int i;

  sprintf(lab1,"%s ",ftran);
  sprintf(lab2,"%s ",ffield);
  sprintf(lab3,"%s ",plot_file);
  if(lnum<3)
  {
    list=win_make(4,6,74,4,STD_FRAME,"",LRED,C_LIST);
    win_print(work,4,9," FILES: TRAN-");
    win_print(work,33,9," FIELD-");
    win_print(work,56,9," PLOT-");
    scr.current=C_ALT;
    win_print(work,17,9,lab1);
    win_print(work,40,9,lab2);
    win_print(work,62,9,lab3);
  }
  else
  {
    list=win_make(4,6,74,1+lnum,STD_FRAME,"",LRED,C_LIST);
    win_print(work,4,6+lnum," FILES: TRAN- ");
    win_print(work,33,6+lnum," FIELD- ");
    win_print(work,56,6+lnum," PLOT- ");
    scr.current=C_ALT;
    win_print(work,17,6+lnum,lab1);
    win_print(work,40,6+lnum,lab2);
    win_print(work,62,6+lnum,lab3);
  }
  sprintf(lab4," %s ",fproc);
  win_print(work,36,2,lab4);
  scr.current=LRED;
  print(2,1,"Layer     K'        K''      [mS/m] Thickness [m]");
  print(58,1,"t (ns):");
  print(55,2,"first (ns):");
  print(56,3,"last (ns):");
  print(54,4,"offset (ns):");
  scr.current=C_WIN_BOR;
  for(i=1;i<=lnum-1;i++)
  {
    sprintf(lab,"%d%10.3f%10.3f%11.3f%13.3f",i,e[i].r,e[i].i,con[i]*1000.,dis[i]);
    print(4,1+i,lab);
  }
  sprintf(lab,"%d%10.3f%10.3f%11.3f",i,e[i].r,e[i].i,con[i]*1000.);
  print(4,1+lnum,lab);
  sprintf(lab,"%8.3f",sample);
  print(66,1,lab);
  sprintf(lab,"%8.3f",first*sample);
  print(66,2,lab);
  sprintf(lab,"%8.3f",last*sample);
  print(66,3,lab);
  sprintf(lab,"%8.3f",offset*sample);
  print(66,4,lab);
}

/*************************************************************************/
void file_not_good(int option)   /* File not found window */
{
  win_make(21,9,36,1,DBL_FRAME,"",RED,WHITE);
  scr.current=YELLOW;
  printf("\a");
  cursor(NO_CURSOR);
  if(option==1)
  {
    print(6,1,"MODELING FILE NOT FOUND");
    win_cls(help);
    scr.current=LGREEN;
    win_print(help,3,2,"Modeling parameter file not present.");
  }
  if(option==2)
  {
    print(10,1,"NOT A VALID FILE");
    win_cls(help);
    scr.current=LGREEN;
    win_print(help,3,2,"File not a valid modeling parameter file.");
  }
  if(option==3)
  {
    print(8,1,"FIELD FILE NOT FOUND");
    win_cls(help);
    scr.current=LGREEN;
    win_print(help,3,2,"Field file not present.");
  } 
  if(option==4)
  {
    print(5,1,"TRANSMISSION FILE NOT FOUND");
    win_cls(help);
    scr.current=LGREEN;
    win_print(help,3,2,"Transmission file not present.");
  }
  if(option==5)
  {
    print(10,8,"RAW FIELD FILE NOT FOUND");
    win_cls(help);
    scr.current=LGREEN;
    win_print(help,3,2,"Raw field file not present.");
  }
  if(option==6)
  {
    print(10,1,"RANGE GAIN FILE NOT FOUND");
    win_cls(help);
    scr.current=LGREEN;
    win_print(help,3,2,"Range gain file not present.");
  }
  if(option==7)
  {
    print(10,10,"TIME CALIBRATION FILE NOT FOUND OR NO GOOD");
    win_cls(help);
    scr.current=LGREEN;
    win_print(help,3,2,"Time calibration file not present or not pickable.");
  }
  if(option==8)
  {
    print(5,1,"NO SAMPLE RATE FOR THIS DATA");
    win_cls(help);
    scr.current=LGREEN;
    win_print(help,3,1,"Transmission file should be 513 8-byte values where the last");
    win_print(help,3,2,"one is the sample rate.  This dataset has no sample rate so ");
    win_print(help,3,3,"it has been set to 0.2 nanoseconds.  TIMES CANNOT BE TRUSTED!");
  }
  if(option==9)
  {
    print(5,1,"NO SAMPLE RATE FOR THIS DATA");
    win_cls(help);
    scr.current=LGREEN;
    win_print(help,3,1,"Calibrated file should be 513 8-byte values where the last");
    win_print(help,3,2,"one is the sample rate.  This dataset has no sample rate so ");
    win_print(help,3,3,"it has been set to match the pulse.  TIMES CANNOT BE TRUSTED!");
  }
  key_win(0);
  getch();
  win_delete_top();
}

/*************************************************************************/
int line_editor(int ans_col,int ans_row,int max_column,int color, \
	char *answer_str,int isfile,int isnumber)
{
  int lstr,icr,ins,itemp,it,ic,i,icl,ir;
  int key=0;
  char lbuffer[40],ibuffer[40];
  unsigned char ickey, attr;

  scr.current=color;
  gotoxy(ans_col,ans_row);
  lstr=strlen(answer_str);
  if(*(answer_str+lstr-1)!=32)
  {
    *(answer_str+lstr++)=32;
    *(answer_str+lstr)=NULL;
  }
  print(ans_col,ans_row,answer_str);
  icl=ans_col;
  ir=ans_row;
  icr=icl+lstr-1;                /* current rightmost column (blank) */
  itemp=icr;                     /* current cursor column position */
  ins= -1;                       /* -1==overtype;  1==insert */
  gotoxy(itemp,ir);
  ceol(itemp,ir);
  cursor(NORMAL_CURSOR);

  while((key=getch())!=RETURN && key!=ESC) /* <Enter> or <Esc> or <F10> */
  {
    if(key==0 || key==0xE0)  key=-getch(); 
    if(key==(-68)) break;
    switch (key)
    {
	 case -82:                  /* insert toggle */
	   ins= -ins;
	   if (ins==-1) cursor(NORMAL_CURSOR);
	   if (ins==1) cursor(BIG_CURSOR);
	   break;
	 case -75:                  /* left arrow */
	   if(--itemp<icl) itemp=icl;
	   break;
	 case -77:                  /* right arrow */
	   if(++itemp>icr) itemp=icr;
	   break;
	 case -71:                  /* home */
	   itemp=icl;
	   break;
	 case -79:                  /* end */
	   itemp=icr;
	   break;
	 case -83:                  /* delete */
	   if (itemp<icr)
	   {
		for(it=itemp+1;it<=icr;it++)
		{
		  gotoxy(it,ir);
		  readxy(&ickey,&attr);
		  sprintf(lbuffer,"%c",ickey);
		  gotoxy(it-1,ir);
		  print_here(lbuffer);
		}
		gotoxy(icr,ir);
		print_here(" ");
		if(--icr<icl) icr=icl;
	   }
	   break;
	 case 8:                    /* backspace */
	   if(itemp>icl)
	   {
		for(it=itemp;it<=icr;it++)
		{
		  gotoxy(it,ir);
		  readxy(&ickey,&attr);
		  sprintf(lbuffer,"%c",ickey);
		  gotoxy(it-1,ir);
		  print_here(lbuffer);
		}
		gotoxy(icr,ir);
		print_here(" ");
		if(--icr<icl) icr=icl;
		itemp--;
	   }
	   break;
	 default:

	 if((isfile==TRUE && ((key>63 && key<91) || (key>96 && key<124) || (key>47 && key<59) \
		 || (key>33 && key<42 ) || key==45 || key==94 || key==125 || key==126 || key==92 \
		 || key==95)) || (isfile==FALSE && isnumber==FALSE && key>31 && key< 127) \
		 || (isfile==FALSE && isnumber==TRUE && ((key>47 && key< 58) || key==46 \
		 || key==32 || key==45 || key==43)))
	 {
	   if(isfile==TRUE && (key>96 && key<123)) key-=32;   /* convert to CAPS */
	   sprintf(lbuffer,"%c\0",key);
	   if((ins<0 && itemp<max_column) || (ins>0 && icr<max_column))
	   {
		if(ins>0)
		{
		  for(it=icr++;it>itemp;it--)
		  {
		    gotoxy(it-1,ir);
		    readxy(&ickey,&attr);
		    gotoxy(it,ir);
		    sprintf(ibuffer,"%c",ickey);
		    print_here(ibuffer);
		  }
		  gotoxy(icr,ir);
		  print_here(" ");
		}
		if(itemp==icr)
		{
		  gotoxy(++icr,ir);
		  print_here(" ");
		}
		gotoxy(itemp++,ir);
		print_here(lbuffer);
	   } else printf("\a");     /* can't do */
	 } else printf("\a");
    }
    gotoxy(itemp,ir);
  }
  if(key!=ESC && key!=(-68))
  {
    *(answer_str+0)=NULL;
    cursor(NO_CURSOR);
    for(i=icl;i<icr;i++)
    {
	 gotoxy(i,ir);
	 readxy(&ickey,&attr);
	 sprintf(lbuffer,"%c\0",ickey);
	 strcat(answer_str,lbuffer);
    }
    gotoxy(itemp,ir);
  }
  if (key==(-68)) key=F10;
  return key;
}

/*************************************************************************/
void plot_info()                 /* information windows */
{
  char key;
  int over=FALSE;
  FILE *tmp_file;

  if(mode==FALSE)
  {

    do
    {
	 wplot=win_make(22,9,37,2,STD_FRAME,"",YELLOW,LCYAN);
	 print(14,1," PLOT ");
	 win_cls(help);
	 win_print(help,3,1,"Enter HPGL plot output file or 'CRT' to write to the screen only.");
	 win_print(help,3,2,"Do not include file extension.");
	 win_print(help,3,3,"File will be saved with '.PLT' extension.");
	 key_win(2);
	 scr.current=C_MAIN;
	 print(7,2,"Plot File ==>");
	 strcpy(plot_file,"CRT\0");
	 key=line_editor(22,2,58,LCYAN,plot_file,TRUE,TRUE);
	 if(key==ESC || key==F10 || strcmp(plot_file,"CRT")==0) {strcpy(plot_file,"CRT\0"); break;}
	 strcat(plot_file,".PLT");
	 win_delete(wplot);
	 if((tmp_file=fopen(plot_file,"rb"))!=NULL) /* Check to see if file exists */
	 {
	   fclose(tmp_file);
	   key_win(1);
	   overw=win_make(22,9,37,1,STD_FRAME,"",WHITE,LRED);
	   print(1,1,"OVERWRITE FILE:   No    Yes");
	   scr.current=BOLD; 
	   print(25,1,"Y");
	   scr.current=scr.inverse;
	   print(18,1,"<No>");
	   win_cls(help);
	   win_print(help,3,2,"Do not overwrite current existing file.");
	   win_redraw_all();

	   while((key=getch())!=RETURN && key!=ESC)
	   {
		if(key==0 || key==0xE0) key=getch();
		if(key==F10) break;
		if(key==78 || key==110) {over=FALSE; break;}
		if(key==89 || key==121) {over=TRUE; key=RETURN; break; }
		if(key==LEFT || key==RIGHT)
		{
		  if(key==LEFT ) if(over==TRUE) over--;
		  if(key==RIGHT) if(over==FALSE) over++;
		  scr.current=W_SAVE;
		  print(1,1,"OVERWRITE FILE:   No    Yes ");
		  scr.current=BOLD; 
		  print(25,1,"Y");
		  print(19,1,"N");
		  scr.current=scr.inverse;
		  if(over==TRUE)
		  {
		    print(24,1,"<Yes>");
		    win_cls(help);
		    win_print(help,3,2,"Overwrite existing file.");
		  }
		  if(over==FALSE)
		  {
		    print(18,1,"<No>");
		    win_cls(help);
		    win_print(help,3,2,"Do not overwrite existing file.");
		  }
		  win_redraw_all();
		}
		win_redraw_all();
	   }

	   win_delete(overw);
	   if(over==TRUE && key==RETURN) break;     /* overwrite file */
	 }
	 else break;
    } while(TRUE);

  }
  else
  {
    wplot=win_make(22,9,37,2,STD_FRAME,"",YELLOW,LCYAN);
    print(1,1," PLOT ");
    printf("\a");
    print(1,2,"Toggle Vary off to create plot file.");
    key_win(0);
    getch();
  }
  win_delete(wplot);
}

/*************************************************************************/
void vary_toggle()               /* turn on & off interactive mode */
{
  scr.current=C_ALT;
  if(mode)
  {
    mode=FALSE;
    scr.current=BOLD;
    win_print(work,47,1,"OFF ");
    win_cls(help);
    scr.current=LGREEN;
    win_print(help,3,2,"Vary parameters in graphics mode is off.");
    vary=win_make(22,9,37,2,STD_FRAME,"",YELLOW,LCYAN);
    cursor(NO_CURSOR);
    print(1,1," VARY ");
    print(1,2,"Plot output may now be changed.");
  }
  else
  {
    mode=TRUE;
    strcpy(plot_file,"CRT\0");
    scr.current=BOLD;
    win_print(work,47,1,"ON ");
    scr.current=C_WIN_BOR;
    print(54,1,"-");
    win_cls(help);
    scr.current=LGREEN;
    win_print(help,3,1,"Vary parameters in graphics mode is on.");
    win_print(help,3,2,"Plot output is set to 'CRT'.");
    vary=win_make(22,9,40,9,STD_FRAME,"",YELLOW,LCYAN);
    cursor(NO_CURSOR);
    print(12,1," VARY ");
    print(2,2,"Increment values in graphics mode.");
    print(2,3,"   chooses a layer value.");
    print(2,4,"Values are incremented as follows:");
    print(2,5,"<Home>  +1.00, <End>   -1.00,");
    print(2,6,"<>     +0.10, <>     -0.10,");
    print(2,7,"<Pg Up> +0.01, <Pg Dn> -0.01,");
    print(2,8,"<Space> or <Return> begins plot");
    print(2,9,"and <Esc> exits.");
  }
  key_win(0);
  getch();
  win_delete(vary);
  if(mode) win_redraw_all();
}

/*************************************************************************/
void options()   /* turn on & off optional calculations */
{
  char key;

  scr.current=C_ALT;
  win_cls(help);
  scr.current=LGREEN;
  win_print(help,3,1,"ON/OFF Toggle for the calculation of 3 multiple reflections events,");
  win_print(help,3,2,"layer center frequency attenuation, and normalized power difference");
  win_print(help,3,3,"between field scan and forward model.");
  key_win(4);
  option=win_make(22,9,27,5,STD_FRAME,"",YELLOW,LCYAN);
  cursor(NO_CURSOR);
  print(10,1," OPTIONS ");
  print(2,2,"Calculate:");
  print(2,3," ultiple reflections:");
  print(2,4," ayer attenuation:");
  print(2,5," ower difference:");
  scr.current=TOGGLE;
  print(2,3,"M");
  print(2,4,"L");
  print(2,5,"P");
  if(aa1==TRUE) {print(24,3,"ON");}
  else print(24,3,"OFF");
  if(aa2==TRUE) {print(24,4,"ON");}
  else print(24,4,"OFF");
  if(aa3==TRUE) {print(24,5,"ON");}
  else print(24,5,"OFF");
  do
  {
    key=getch();
    if(key==0 || key==0xE0) key=getch();
    if(key==109 || key==77)
    {
	 print(24,3,"   ");
	 if(aa1==TRUE)
	 {
	   aa1=FALSE;
	   print(24,3,"OFF");
	 }
	 else
	 {
	   aa1=TRUE;
	   print(24,3,"ON");
	 }
    }
    if(key==108 || key==76)
    {
	 print(24,4,"   ");
	 if(aa2==TRUE)
	 {
	   aa2=FALSE;
	   print(24,4,"OFF");
	 }
	 else
	 {
	   aa2=TRUE;
	   print(24,4,"ON");
	 }
    }
    if(key==80 || key==112)
    {
	 print(24,5,"   ");
	 if(aa3==TRUE)
	 {
	   aa3=FALSE;
	   print(24,5,"OFF");
	 }
	 else
	 {
	   aa3=TRUE;
	   print(24,5,"ON");
	 }
    }
  } while(key!=ESC && key!=F10 && key!=RETURN);

  win_delete(option);
}

/*************************************************************************/
int check_file(FILE *p_file)     /* is requested .MOD file present? */
{
    char tmp[15];
    int i;

    fgets(tmp,15,p_file);
    rewind(p_file);
    return(strcmp(tmp,"Plot title ==>"));
}

/*************************************************************************/
void check_quit()  /* If F10 is pressed, exit to DOS window is prompted */
{
  char key;
  int i,j,ex=TRUE;

  key_win(1);
  wexit=win_make(22,9,37,1,DBL_FRAME,"",RED,LCYAN);
  cursor(NO_CURSOR);
  scr.current=C_MAIN;
  print(6,1,"EXIT TO DOS:   Yes    No ");
  scr.current=BOLD; 
  print(28,1,"N");
  scr.current=scr.inverse;
  print(20,1,"<Yes>");
  win_cls(help);
  scr.current=LGREEN;
  win_print(help,3,2,"Exit program and return to DOS.");

  while((key=getch())!=RETURN && key!=ESC)
  {
    if(key==0 || key==0xE0) key=getch();
    if(key==F10) break;
    if(key==78 || key==110) {ex=FALSE; break;}
    if(key==89 || key==121) {ex=TRUE; break;}
    if(key==LEFT || key==RIGHT)
    {
	 if(key==LEFT ) if(ex==FALSE) ex++;
	 if(key==RIGHT) if(ex==TRUE) ex--;
	 scr.current=C_MAIN;
	 print(6,1,"EXIT TO DOS:   Yes    No ");
	 scr.current=BOLD; 
	 print(21,1,"Y");
	 print(28,1,"N");
	 scr.current=scr.inverse;
	 if(ex==TRUE)
	 {
	   print(20,1,"<Yes>");
	   win_cls(help);
	   scr.current=LGREEN;
	   win_print(help,3,2,"Exit program and return to DOS.");
	 }
	 if(ex==FALSE)
	 {
	   print(27,1,"<No>");
	   win_cls(help);
	   scr.current=LGREEN;
	   win_print(help,3,2,"Return to forward modeling program.");
	 }
	 win_redraw_all();
    }
    win_redraw_all();
  }

  if(key==ESC || key==F10 || ex==FALSE) {win_delete(wexit);}
  if(ex==TRUE && key!=ESC && key!=F10)
  {
    win_delete(wexit);
    if(check_save()) save_file(1); /* Check if changes to file have been made */
    restore_cur();
    exit(0);
  }
}

/*************************************************************************/
void restore_cur() /* Restore cursor to original size and clear screen */
{
  win_delete(wtitle);win_delete(work);win_delete(list);
  win_delete(help);win_delete(keys);win_delete(change);
  cursor(NORMAL_CURSOR);  /* Restore cursor */
  scr.current=LWHITE;
}

/*************************************************************************/
void plot_setup(char *title,int pltfile,int g_mode)    /* draw plot bkgrnd */
{
  char lab[50];
  int i,step;
  double mx;

  plotter_is(g_mode,plot_file);
  viewport(0.,100.,95.,100.);
  window(0.,100.,0.,100.);
  pen(15);                       /* white */
  lorg(5);
  csize(1.1);
  label(50.,50.,title);
  mx=(last-first)*sample;
  if(mx>=600.) step=100;
  else if(mx>=300.) step=50;
  else if(mx>=125.) step=25;
  else if(mx>=60.) step=10;
  else if(mx>=30.) step=5;
  else if(mx>=12.) step=2;
  else step=1;
  viewport(0.,100.,0.,8.);
  window(first*sample,last*sample,0.,10.);
  lorg(1);
  csize(.7);
  for(i=0;i<=(int)((last+.05)*sample);i+=step)
  {
    plot((double)(i),10.,-2);
    plot((double)(i),8.,-1);
    sprintf(lab,"%d",i);
    label((double)i,7.,lab);
  }
  lorg(6);
  csize(1.);
  label(mx/2.+(first*sample),0.5,"Two-Way Travel Time (ns)");
  unclip();
  viewport(0.,100.,8.,95.);
  window(0.,100.,0.,100.);
  frame();
  lorg(8);
  pen(12);                       /* red */
  label(95.,90.,"Field Scan");
  if(pltfile)
  {
    linetype(2);
    plot(62.,90.,-2);
    plot(71.6,90.,-1);
  }
  pen(14);                       /* yellow */
  label(95.,96.,"Forward Model");
  if(pltfile)
  {
    linetype(0);
    plot(57.,96.,-2);
    plot(67.,96.,-1);
  }
}

/*************************************************************************/
void plot_layer()      /* draw layer model on right side of screen */
{
  char lab[50];
  int i;
  double mx;

  for(i=1;i<lnum;i++) d[i]=d[i-1]+dis[i];
  mx=d[lnum-1]*1.5;
  viewport(101.,133.,0., 100.);
  window(0.,100.,mx,-mx/20.);
  pen(9); /* blue */
  plot(0.,0.,-2);
  plot(100.,0.,-1);
  for(i=1;i<lnum;i++)
  {                       
    plot(0.,d[i],-2);
    plot(75.,d[i],-1);
  }
  pen(7); /* dim white */
  plot(0.,d[lnum-1]*1.4,-2);
  plot(100.,d[lnum-1]*1.4,-1);
  pen(15);
  csize(.75);
  lorg(8);
  for(i=1;i<lnum;i++)
  {
    sprintf(lab,"%5.2f",(float)d[i]);
    label(100.,d[i],lab);
  }
  label(50.,-mx/40.,"Surface");
  for(i=1;i<lnum;i++)
  {
    lorg(2);
    sprintf(lab,"K'=%5.2f;",(float)e[i].r);
    label(2.,d[i-1]+dis[i]/3.,lab);
    sprintf(lab,"con=%6.2f mS/m",(float)con[i]*1000.);
    label(2.,d[i-1]+2.*dis[i]/3.,lab);
    lorg(8);
    sprintf(lab,"K''=%5.2fj",(float)e[i].i);
    label(98.,d[i-1]+dis[i]/3.,lab);
  }
  lorg(2);
  sprintf(lab,"K'=%5.2f;",(float)e[lnum].r);
  label(2.,d[lnum-1]+(mx-d[lnum-1])/3.,lab);
  sprintf(lab,"con=%6.2f mS/m",(float)con[lnum]*1000.);
  label(2.,d[lnum-1]+(mx-d[lnum-1])/2.,lab);
  lorg(8);
  sprintf(lab,"K''=%5.2fj",(float)e[lnum].i);
  label(98.,d[lnum-1]+(mx-d[lnum-1])/3.,lab);
  /************* print offset and Cr at bottom of model ************/
  lorg(2);
  sprintf(lab,"Offs=%5.2f",(float)(offset*sample));
  label(2.,(0.967*mx),lab);
  lorg(8);
  sprintf(lab,"Cr=%5.2f",(float)c_ratio);
  label(98.,(0.967*mx),lab);
}

/*************************************************************************/
int increment_layer(double *dis_tmp)   /* interactive param change */
{
  char lab[50],tmp[50],key;
  int i;
  double mx,inc,noffset;
  mx=d[lnum-1]*1.5;

  cr_flag=0;
  vinc=0.0;
  viewport(101.,133.,0., 100.);
  window(0.,100.,mx,-mx/20.);
  csize(.75);
  if(l==1 && p==1)
  {
    pen(12); lorg(2);            /* red */
    sprintf(lab,"K'=%5.2f;",(float)e[1].r);
    label(2.,dis_tmp[1]/3.,lab);
  }

  while((key=getch())!=SPACE && key!=RETURN && key!=ESC)
  {
    if(key==0 || key==0xE0) key=getch(); 
    if(key==LEFT)
    {
	 if(p==1 && l>1) {l--; p=4;}
	 else if(p==1 && l==1) {l=lnum; p=5;}
	 else p--;
    }
    if(key==RIGHT)
    {
	 if(p==4 && l<lnum) {l++; p=1;}
	 else if(p==5 && l==lnum) {l=1; p=1;}
	 else p++;
    }
    if(key==PGUP) inc=.01;
    else if(key==PGDN) inc=(-.01);
    else if(key==UP) inc=.1;
    else if(key==DOWN) inc=(-.1);
    else if(key==HOME) inc=1.;
    else if(key==END) inc=(-1.);
    else inc=0.;
    if(p==1)
    {
	 sprintf(tmp,"K'=%5.2f;",(float)e[l].r);
	 e[l].r+=inc;
	 if(e[l].r>100.) {printf("\a"); e[l].r=100.;}
	 else if(e[l].r<1.) {printf("\a"); e[l].r=1.;}
	 sprintf(lab,"K'=%5.2f;",(float)e[l].r);
	 lorg(2); pen(0);
	 if(l==lnum)
	 {
	   label(2.,d[l-1]+(mx-d[l-1])/3.,tmp);
	   pen(12);                 /* red */
	   label(2.,d[l-1]+(mx-d[l-1])/3.,lab);
	   pen(15); lorg(8);        /* white */
	   sprintf(lab,"%5.2f",(float)dd[l-1]);
	   label(100.,d[l-1],lab);
	   sprintf(lab,"K''=%5.2fj",(float)e[l].i);
	   label(98.,d[l-1]+(mx-d[l-1])/3.,lab);
	 }
	 else if(l==1)
	 {
	   label(2.,dis_tmp[1]/3.,tmp);
	   pen(12);                 /* red */
	   label(2.,dis_tmp[1]/3.,lab);
	   lorg(8); pen(15);        /* white */
	   sprintf(lab,"Cr=%5.2f",(float)c_ratio);
	   label(98.,(0.967*mx),lab);
	   sprintf(lab,"K''=%5.2fj",(float)e[l].i);
	   label(98.,d[l-1]+dis_tmp[l]/3.,lab);
	 }
	 else
	 {
	   label(2.,d[l-1]+dis_tmp[l]/3.,tmp);
	   pen(12);                 /* red */
	   label(2.,d[l-1]+dis_tmp[l]/3.,lab);
	   pen(15); lorg(8);        /* white */
	   sprintf(lab,"%5.2f",(float)dd[l-1]);
	   label(100.,d[l-1],lab);
	   sprintf(lab,"K''=%5.2fj",(float)e[l].i);
	   label(98.,d[l-1]+dis_tmp[l]/3.,lab);
	 }
	 for(i=l;i<lnum;i++)
	 {
	   sprintf(lab,"%5.2f",(float)dd[i]);
	   pen(15);                 /* white */
	   label(100.,d[i],lab);
	 }
    }
    if(p==2)
    {
	 sprintf(tmp,"K''=%5.2fj",(float)e[l].i);
	 e[l].i+=inc;
	 if(e[l].i>100.) {printf("\a"); e[l].i=100.;}
	 else if(e[l].i<0.) {printf("\a"); e[l].i=0.;}
	 sprintf(lab,"K''=%5.2fj",(float)e[l].i);
	 lorg(8); pen(0);
	 if(l==lnum)
	 {
	   label(98.,d[l-1]+(mx-d[l-1])/3.,tmp);
	   pen(12);                 /* red */
	   label(98.,d[l-1]+(mx-d[l-1])/3.,lab);
	   pen(15); lorg(2);        /* white */
	   sprintf(lab,"K'=%5.2f;",(float)e[l].r);
	   label(2.,d[l-1]+(mx-d[l-1])/3.,lab);
	   sprintf(lab,"con=%6.2f mS/m",(float)con[l]*1000.);
	   label(2.,d[l-1]+(mx-d[l-1])/2.,lab);
	 }
	 else
	 {
	   label(98.,d[l-1]+dis_tmp[l]/3.,tmp);
	   pen(12);                 /* red */
	   label(98.,d[l-1]+dis_tmp[l]/3.,lab);
	   pen(15); lorg(2);        /* white */
	   sprintf(lab,"K'=%5.2f;",(float)e[l].r);
	   label(2.,d[l-1]+dis_tmp[l]/3.,lab);
	   sprintf(lab,"con=%6.2f mS/m",(float)con[l]*1000.);
	   label(2.,d[l-1]+2.*dis_tmp[l]/3.,lab);
	 }
    }
    if(p==3)
    {
	 sprintf(tmp,"con=%6.2f mS/m",(float)con[l]*1000.);
	 con[l]+=(inc/1000.);
	 if(con[l]>10.) {printf("\a"); con[l]=10.;}
	 else if(con[l]<0.) {printf("\a"); con[l]=0.;}
	 sprintf(lab,"con=%6.2f mS/m",(float)con[l]*1000.);
	 lorg(2); pen(0);
	 if(l==lnum)
	 {
	   label(2.,d[l-1]+(mx-d[l-1])/2.,tmp);
	   pen(12);                 /* red */
	   label(2.,d[l-1]+(mx-d[l-1])/2.,lab);
	   pen(15);                 /* white */
	   sprintf(lab,"Offs=%5.2f",(float)(offset*sample));
	   label(2.,(0.967*mx),lab);
	   lorg(8);
	   sprintf(lab,"K''=%5.2fj",(float)e[l].i);
	   label(98.,d[l-1]+(mx-d[l-1])/3.,lab);
	 }
	 else
	 {
	   label(2.,d[l-1]+2.*dis_tmp[l]/3.,tmp);
	   pen(12);                 /* red */
	   label(2.,d[l-1]+2.*dis_tmp[l]/3.,lab);
	   pen(15); lorg(8);        /* white */
	   sprintf(lab,"K''=%5.2fj",(float)e[l].i);
	   label(98.,d[l-1]+dis_tmp[l]/3.,lab);
	   sprintf(lab,"%5.2f",(float)dd[l]);
	   label(100.,d[l],lab);
	 }
	 for(i=l;i<lnum;i++)
	 {
	   sprintf(lab,"%5.2f",(float)dd[i]);
	   pen(15);                 /* white */
	   label(100.,d[i],lab);
	 }
    }
    if(p==4)
    {
	 if(l==lnum)
	 {
	   /*********** edit offset ********************/
	   sprintf(tmp,"Offs=%5.2f",(float)(offset*sample));
	   noffset=offset+(inc/sample);
	   if(noffset>511.) {printf("\a"); noffset=511.;}
	   else if(noffset<(-511.)) {printf("\a"); noffset=(-511.);}
	   if((noffset-offset)!=0.0) {vinc+=noffset-offset; offset=noffset;}
	   sprintf(lab,"Offs=%5.2f",(float)(offset*sample));
	   lorg(2); pen(0);         /* black */
	   label(2.,(0.967*mx),tmp);
	   pen(12);                 /* red */
	   label(2.,(0.967*mx),lab);
	   pen(15); lorg(2);        /* white */
	   sprintf(lab,"con=%6.2f mS/m",(float)con[lnum]*1000.);
	   label(2.,d[lnum-1]+(mx-d[lnum-1])/2.,lab);
	   lorg(8);
	   sprintf(lab,"Cr=%5.2f",(float)c_ratio);
	   label(98.,(0.967*mx),lab);
	 }
	 else
	 {
	   dis[l]+=inc;
	   if(dis[l]>500.) {printf("\a"); dis[l]=500.;}
	   else if(dis[l]<0.01) {printf("\a"); dis[l]=0.01;}
	   for(i=l;i<lnum;i++)
	   {
		sprintf(tmp,"%5.2f",(float)dd[i]);
		lorg(8); pen(0);
		label(100.,d[i],tmp);
	   }
	   for(i=1;i<lnum;i++) dd[i]=dd[i-1]+dis[i];
	   for(i=l;i<lnum;i++)
	   {
		sprintf(lab,"%5.2f",(float)dd[i]);
		pen(12);               /* red */
		label(100.,d[i],lab);
	   }
	   pen(15); lorg(2);        /* white */
	   sprintf(lab,"con=%6.2f mS/m",(float)con[l]*1000.);
	   label(2.,d[l-1]+2.*dis_tmp[l]/3.,lab);
	   sprintf(lab,"K'=%5.2f;",(float)e[l+1].r);
	   if(l==lnum-1) label(2.,d[lnum-1]+(mx-d[lnum-1])/3.,lab);
	   else label(2.,d[l]+dis_tmp[l+1]/3.,lab);
	 }
    }
    if(p==5)
    {
	 /*********** edit Cr ********************/
	 sprintf(tmp,"Cr=%5.2f",(float)c_ratio);
	 c_ratio+=inc;
	 if(inc!=0.0) cr_flag=1;
	 if(c_ratio>5.0) {printf("\a"); c_ratio=5.0;}
	 else if(c_ratio<min_cr) {printf("\a"); c_ratio=min_cr;}
	 sprintf(lab,"Cr=%5.2f",(float)c_ratio);
	 lorg(8); pen(0);           /* black */
	 label(98.,(0.967*mx),tmp);
	 pen(12);                   /* red */
	 label(98.,(0.967*mx),lab);
	 pen(15); lorg(2);          /* white */
	 sprintf(lab,"K'=%5.2f;",(float)e[1].r);
	 label(2.,dis_tmp[1]/3.,lab);
	 sprintf(lab,"Offs=%5.2f",(float)offset*sample);
	 label(2.,(0.967*mx),lab);
    }
  }

  return(key);
}

/*************************************************************************/
void plot_scan(double off,double *data,int color)    /* plots data */
{
  int i;

  pen(color);
  plot(off,data[1],-2);
  for(i=2;i<N;i++) plot((double)(i)+off,data[i],-1);
}

/*************************************************************************/
void calc_att_dis(double *data,double delf)
{
  int i,center;
  double mx,power[N+1],w_center;

  mx=0.;
  for(i=1;i<=N/2;i++)
  {
    power[i]=SQR(data[2*i-1])+SQR(data[2*i]);
    if(power[i]>mx)
    {
	 mx=power[i];
	 center=i;
    }
  }
  w_center=2.*PI*(double)(center)*delf;
  for(i=1;i<=lnum;i++) att_dis[i]=Cprop(e[i],con[i],w_center);
}

/*************************************************************************/
void ray_trace(int rl,int im,double real,double imag) /* calc effects thru model */
{
  int j,h;
  complex trace;

  ray[0].r=real;
  ray[0].i=imag;
  for(j=1;j<lnum;j++)
  {
    ray[j]=Ctran(ray[j-1],k[j-1],k[j]);
    ray[j]=Catt(ray[j],k[j],dis[j]);
  }
  for(j=1;j<lnum;j++)
  {
    trace=Cref(ray[j],k[j],k[j+1]);
    for(h=j;h>=1;h--)
    {
	 trace=Catt(trace,k[h],-dis[h]);
	 trace=Ctran(trace,k[h],k[h-1]);
    }
    reflect[j][rl]=trace.r;
    reflect[j][im]=trace.i;
  }
}

/*************************************************************************/
void mult_trace(int rl,int im)   /* optionally calc effects of multiples */
{
  int j,h;
  complex trace;
  /* Calculate first two multiples ***** ---\------/\--/\-----/--/- Surface */
  ray[1]=Cref(ray[1],k[1],k[2]);      /*     \    /  \/  \/  /              */
  ray[2]=Cref(ray[2],k[2],k[3]);      /*      \  /   / \  \/  /     Layer 1 */
  ray[2]=Catt(ray[2],k[2],-dis[2]);   /* ------\/---/----\/\/-------------  */
  for(j=1;j<=2;j++)                   /*        \  /                Layer 2 */
  {                                   /*---------\/----------------------   */
    if(j==1) {trace.r=ray[1].r; trace.i=ray[1].i;}
    else trace=Ctran(ray[2],k[2],k[1]);
    trace=Catt(trace,k[1],-dis[1]);
    for(h=0;h<=2;h+=2)
    {
	 trace=Cref(trace,k[1],k[h]);
	 trace=Catt(trace,k[1],(h-1.)*dis[1]);
    }
    trace=Ctran(trace,k[1],k[0]);
    mult[j][rl]=trace.r;
    mult[j][im]=trace.i;
  }
  /*** Calculate third multiple ***** --\----------------/-------- Surface */
  for(j=1;j<=3;j+=2)               /*    \              /                  */
  {                                /*     \            /           Layer 1 */
    trace=Cref(trace,k[2],k[j]);   /* -----\----/\----/-----------------   */
    trace=Catt(trace,k[2],(j-2.)*dis[2]);/* \  /  \  /             Layer 2 */
  }                                /* -------\/----\/-------------------   */
  trace=Ctran(trace,k[2],k[1]);
  trace=Catt(trace,k[1],-dis[1]);
  trace=Ctran(trace,k[1],k[0]);
  mult[3][rl]=trace.r;
  mult[3][im]=trace.i;
}

/*************************************************************************/
void ray_model()   /* sum reflections to get model data (inverse FFT back to time) */
{
  int i,j;
  double tmp,spread=0.;

  for(i=1;i<=N;i++) model[i]=0.;
  for(i=1;i<lnum;i++)
  {
    realft(*(reflect+i),N/2,-1);    /* Inverse FFT data */
    spread+=2.*dis[i];
    tmp=(double)N/2.*spread;
    for(j=1;j<=N;j++) reflect[i][j]/=(tmp);  /* Scale FFT & correct for spreading losses */
    /********** Add reflections to get model *********************************/
    delay[i]=delay[i-1]+(int)(((2*dis[i]*sqrt(e[i].r))/(sample*Vc)));
    for(j=delay[i]+1;j<=N;j++) model[j]+=reflect[i][j-delay[i]];
 }
}

/*************************************************************************/
void mult_model()  /* inverse FFT and sum multiples into model */
{
  int i,j,mdelay[4];
  double spread,tmp;

  for(i=1;i<=3;i++) realft(*(mult+i),N/2,-1);
  spread=4.*dis[1];
  tmp=(double)N/2.*spread;

  for(i=1;i<=N;i++) mult[1][i]/=(tmp);
  spread=4.*dis[1]+2.*dis[2];
  tmp=(double)N/2.*spread;

  for(i=1;i<=N;i++) mult[2][i]/=(tmp);
  spread=2.*dis[1]+4.*dis[2];
  tmp=(double)N/2.*spread;

  for(i=1;i<=N;i++) mult[3][i]/=(tmp);
  mdelay[1]=2*delay[1];
  mdelay[2]=2*delay[2]-delay[1];
  mdelay[3]=delay[1]+2*delay[2];
  for(i=1;i<=3;i++) for(j=mdelay[i]+1;j<=N;j++)

  model[j]+=mult[i][j-mdelay[i]];
}

/*************************************************************************/
complex Cdiv(complex a,complex b)      /* performs complex division */
{
  complex c;
  double r,den;
  if (fabs(b.r) >= fabs(b.i))
  {
    r=b.i/b.r;
    den=b.r+r*b.i;
    c.r=(a.r+r*a.i)/den;
    c.i=(a.i-r*a.r)/den;
  }
  else
  {
    r=b.r/b.i;
    den=b.i+r*b.r;
    c.r=(a.r*r+a.i)/den;
    c.i=(a.i*r-a.r)/den;
  }
  return c;
}

/*************************************************************************/
complex Cprop(complex e,double con,double w)     /* finds k.r & k.i for a layer */
{
  complex k;
  double loss;
  loss=((e.i+(con/(w*eo)))/e.r);
  k.r=w*sqrt(eo*e.r*uo/2.*(sqrt(1.+(loss*loss))+1.));
  k.i=w*sqrt(eo*e.r*uo/2.*(sqrt(1.+(loss*loss))-1.));
  return k;
}

/*************************************************************************/
complex Catt(complex c,complex k,double dist)    /* find attenuation due to dist */
{
  complex a;
  a.r=exp(-k.i*fabs(dist))*((c.r*cos(k.r*dist))-(c.i*sin(k.r*dist)));
  a.i=exp(-k.i*fabs(dist))*((c.i*cos(k.r*dist))+(c.r*sin(k.r*dist)));
  return a;
}

/*************************************************************************/
complex Cref(complex c,complex k1,complex k2) /* find effect of boundary reflection */
{
  complex t1,t2,t3,r;
  t1.r=k1.r-k2.r;
  t1.i=k1.i-k2.i;
  t2.r=Cradd(k1,k2);
  t2.i=Ciadd(k1,k2);
  t3=Cdiv(t1,t2);
  r.r=Crmul(t3,c);
  r.i=Cimul(t3,c);
  return r;
}

/*************************************************************************/
complex Ctran(complex c,complex k1,complex k2)   /* find effect of boundary transmission */
{
  complex t1,t2,t3,t;
  t1.r=2.*k1.r;
  t1.i=2.*k1.i;
  t2.r=Cradd(k1,k2);
  t2.i=Ciadd(k1,k2);
  t3=Cdiv(t1,t2);
  t.r=Crmul(t3,c);
  t.i=Cimul(t3,c);
  return t;
}

/*************************************************************************/
void resample(int option,double *sdata,double old_samp,double new_samp,int g_mode)
{
  char win[500],key,char_sample[80],lab[80];
  int i,j,n,m,cnt,p,rl,im,max_i;
  double new_dtdata[N+1],x[N+1],y2[N+1],new_x,mn,mx,max_f,ty,y,nyq;
  double tm,max_t,plt_digi,fdata[N+1],space,xloc,factor,power_spec[(N/2)+1];

  if(option)
  {
    resam=win_make(22,9,37,2,DBL_FRAME,"",YELLOW,LCYAN);
    print(14,1," COUPLING RATIO ");
    win_cls(help);
    scr.current=LGREEN;
    sprintf(lab,"Enter a value [%5.3f - 5.0] to stretch/compress the transmitted",min_cr);
    win_print(help,3,1,lab);
    win_print(help,3,2,"pulse to simulate coupling changes in frequency content.  The");
    win_print(help,3,3,"higher the coupling ratio (Cr) the more stretched the pulse.");
    key_win(2);
    scr.current=LCYAN;
    print(2,2,"   Enter Cr ==>");
    gcvt(c_ratio,2,char_sample);
    strcat(char_sample,"\0");

    do
    {
	 key=line_editor(19,2,58,LCYAN,char_sample,FALSE,TRUE);
	 if(key==ESC || key==F10) {win_delete(resam);return;}
	 if(char_sample=="" || char_sample=="\0") {win_delete(resam);return;}
	 c_ratio=atof(char_sample);
	 if(c_ratio<min_cr || c_ratio>5.) printf("\a");
	 else break;
    } while(TRUE);

    new_samp/=c_ratio;
    win_delete(resam);
  }
  ratio=old_samp/new_samp;
  if (option || cr_flag) for(i=1;i<=N;i++) sdata[i]=dtdata_in[i];
  for(i=1;i<=N;i++) x[i]=(double)((i-1)*old_samp);

  /********** Eliminate freqs. above new Nyquist if necessary ************/
  if (ratio<1.0)
  {
    /*********** Forward FFT data to freq domain *********************/
    realft(sdata,N/2,1);
    if(option) { for(i=1;i<=N;i++) fdata[i]=sdata[i]; }

    j=(int)((N/2)*ratio);        /* index # of Nyquist freq */
    m=2*(FRAMP-1);               /* full Hanning window size */
    for (i=j-(int)(m*.5);i<=j;i++)     /* for 1/2 window just below Nyquist */
    {
	 n=i+m-j;                   /* Hanning ramp function */
	 sdata[2*i-1]*=0.5-(0.5*cos(2.*PI*(double)n/(double)m));   
	 sdata[2*i]*=0.5-(0.5*cos(2.*PI*(double)n/(double)m));
    }  
    for(i=j+1;i<=N/2;i++)        /* zero everything above Nyquist freq */
    {
	 sdata[2*i-1]=0.;
	 sdata[2*i]=0.;
    }
    /*********** Inverse FFT data and scale FFT **************************/
    realft(sdata,N/2,-1);
    for(i=1;i<=N;i++) sdata[i]*=(2./(double)N);
  }

  /*********** Resample FFT at new rate using spline interpolation *******/
  spline(x,sdata,N,1.e33,1.e33,y2);
  cnt=0;
  for(i=1;i<=(int)(N*ratio);i++)
  {
    if(i>N) break;
    cnt++;
    new_x=((double)(i-1)*new_samp);
    splint(x,sdata,y2,N,new_x,&new_dtdata[i]);
  }
  for(i=cnt+1;i<=N;i++) new_dtdata[i]=0.;
  for(i=1;i<=N;i++) sdata[i]=new_dtdata[i];

  /****************************** GRAPHICS *******************************/
  if(option)
  {
    if(ratio>=1.0) 
    {
	 for(i=1;i<=N;i++) fdata[i]=dtdata_in[i];
	 realft(fdata,N/2,1); 
    }
    for(i=1;i<=N/2;i++) 
    {
	 rl=(2*i)-1; im=2*i; 
	 power_spec[i]=(fdata[rl]*fdata[rl])+(fdata[im]*fdata[im]); 
    }
    old_samp=t_sample; new_samp=old_samp*c_ratio;

    /******* Determine window limits *************************************/
    mn=9999.; mx= -9999.;
    for(i=1;i<=150;i++)
    {
	 mn=MIN(mn,dtdata_in[i]);
	 mx=MAX(mx,dtdata_in[i]);
    }
    if(c_ratio<1.0) {plt_digi=old_samp;} else {plt_digi=new_samp;}
    max_t=150.*plt_digi; 
    plotter_is(g_mode,plot_file);
    viewport(3.,100.,55.,95.);
    window(plt_digi,max_t,mn+mn/5.,mx+mx/5.);
    pen(15);                     /* white */
    frame(); clip(); 
    /****************** plot original pulse in white ***********/
    plot(plt_digi,dtdata_in[1],-2);
    for(i=2;i<=150;i++) {tm=old_samp*i; plot(tm,dtdata_in[i],-1);}
    /****************** plot new pulse in yellow ****************/
    pen(14);                     /* yellow */
    plot(plt_digi,dtdata_in[1],-2); linetype(2);
    for(i=2;i<=150;i++) {tm=new_samp*i; plot(tm,dtdata_in[i],-1);}
    /****************** label time axis *******************************/
    if(max_t>10000.) factor=10000.;
    else if(max_t>1000.) factor=1000.;
    else if(max_t>100.) factor=100.;
    else if(max_t>10.) factor=10.;
    else if(max_t>1.) factor=1.;
    else factor=0.1;
    p=(int)(max_t/factor);
    switch (p) 
    {
	 case 1: space=factor/10.; break;
	 case 2: space=factor/5.; break;
	 case 3: space=factor/5.; break;
	 case 4: space=factor/2.; break;
	 case 5: space=factor/2.; break;
	 default: space=factor; break;
    }
    viewport(3.,100.,45.,55.);
    window(plt_digi,max_t,0.,10.);
    lorg(4); csize(0.5); pen(15); unclip(); /* white */
    label(plt_digi,8.,"0"); 
    plot(plt_digi,9.,-2); plot(plt_digi,10.,-1);
    for(i=1;(double)i*space<=max_t;i++) 
    {
	 xloc=i*space;
	 plot(xloc,9.,-2); plot(xloc,10.,-1);
	 sprintf(lab,"%5.1f",xloc);
	 label(xloc,8.,lab); 
    }
    /****************** label at top of graph ******************/
    viewport(3.,100.,95.,100.);
    window(0.,100.,0.,100.);
    pen(15); lorg(5); csize(1.4); /* white */
    label(50.,50.,"Transmitted Pulse Signatures");
    /****************** side label *****************************/
    viewport(101.,130.,0.,95.);
    window(0.,30.,0.,100.);
    pen(15); plot(2.,90.,-2); linetype(0); plot(10.,90.,-1);
    lorg(2); csize(1.); 
    label(12.,90.,"Cr=1.00"); 
    pen(14); plot(2.,80.,-2); linetype(2); plot(10.,80.,-1);
    label(12.,80.,"Cr=");
    sprintf(lab,"%5.2f",c_ratio);
    label(16.5,80.,lab); 
    pen(15); csize(1.2); label(2.5,57.5,"Time (ns)");
    label(2.5,12.,"Frequency");
    label(5.0,8.5,"(MHz)");
    csize(1.3); label(5.,39.,"Power"); label(5.,35.,"Spectra");
    /********* Power spectra plot ***********************************/
    new_samp=1000./((double)N*new_samp); old_samp=1000./((double)N*old_samp); 
    if(c_ratio<1.0) {plt_digi=new_samp;} else {plt_digi=old_samp;}
    max_f=plt_digi*(double)(N/2);
    mn=1e32; mx=-1e32;
    for(i=1;i<=N/2;i++)
    {
	 if(power_spec[i]>mx) { mx=power_spec[i]; max_i=i; }
	 if(power_spec[i]<mn) mn=power_spec[i];
    }
    viewport(3.,100.,10.,45.);
    window(plt_digi,max_f,mn+mn/5.,mx+mx/5.);
    linetype(0); pen(15);        /* white */
    frame(); clip(); 
    /****************** plot original spectrum in white *********/
    plot(plt_digi,power_spec[1],-2);
    for(i=2;i<=N/2;i++) {tm=old_samp*i; plot(tm,power_spec[i],-1);}
    /****************** plot new spectrum in yellow ****************/
    pen(14);                     /* yellow */
    plot(plt_digi,power_spec[1],-2); linetype(2);
    for(i=2;i<=N/2;i++) {tm=new_samp*i; plot(tm,power_spec[i],-1);}
    /****************** draw Nyquist cutoff frequency **************/
    nyq=1000./(2.*sample);
    if (nyq<max_f)
    {
	 pen(13);                   /* l. magenta */
	 plot(nyq,0.,-2); plot(nyq,mx+mx/5.,-1);
	 ty=1.2*(mx-mn);
	 for(i=0;i<10;i++) 
	 {
	   y=(double)i*.1*ty;
	   plot(nyq,y,-2); plot(max_f,y,-1); 
	 }
    }
    /****************** label max freq *****************************/
    pen(14); lorg(3); tm=max_i*new_samp;  
    plot(tm,0.,-2); plot(tm,mx,-1);
    sprintf(lab,"%5.0f",tm); 
    csize(0.5); label(tm,mx*1.05,lab); 
    tm=max_i*old_samp; linetype(0); pen(15);
    plot(tm,0.,-2); plot(tm,mx,-1);
    sprintf(lab,"%5.0f",tm);
    csize(0.5); label(tm,mx*.95,lab); 
    /****************** label frequency axis **********************/
    if(max_f>10000.) factor=10000.;
    else if(max_f>1000.) factor=1000.;
    else if(max_f>100.) factor=100.;
    else if(max_f>10.) factor=10.;
    else if(max_f>1.) factor=1.;
    else factor=0.1;
    p=(int)(max_f/factor);
    switch (p) 
    {
	 case 1: space=factor/10.; break;
	 case 2: space=factor/5.; break;
	 case 3: space=factor/5.; break;
	 case 4: space=factor/2.; break;
	 case 5: space=factor/2.; break;
	 default: space=factor; break;
    }
    viewport(3.,100.,0.,10.);
    window(plt_digi,max_f,0.,10.);
    linetype(0); lorg(4); csize(0.5); pen(15); unclip(); /* white */
    label(plt_digi,8.,"0"); 
    plot(plt_digi,9.,-2); plot(plt_digi,10.,-1);
    for(i=1;(double)i*space<=max_f;i++) 
    {
	 xloc=i*space;
	 plot(xloc,9.,-2); plot(xloc,10.,-1);
	 sprintf(lab,"%5.0f",xloc);
	 label(xloc,8.,lab); 
    }
    if (nyq<max_f)
    {
	 csize(0.8); lorg(2); pen(13);    /* l. magenta */
	 label(nyq,5.,"Nyq");
    }
    
    getch();
    plotter_off();
    win_redraw_all();
    ALT_win(1);    /* Display ALT options on top of main window */
  }
}

/*************************************************************************/
void import_field(int option)    /* input .CAL file */
{
  char file[80];
  int key,i,k;
  FILE *f_field;

  if(option)
  {
    if((strcmp(ffield,"UNTITLED"))!=0)
    {
	 strcpy(file,ffield);
	 k=strlen(file);
	 file[k-4]='\0';
    }
    else strcpy(file,"\0");

    do
    {
	 win_cls(help);
	 scr.current=LGREEN;
	 win_print(help,3,1,"Enter name of a previously saved and gain-corrected field file.");
	 win_print(help,3,2,"Do not include file extension.");
	 win_print(help,3,3,"File should be saved with '.CAL' extension.");
	 key_win(2);
	 wfield=win_make(22,9,37,2,STD_FRAME,"",YELLOW,C_MAIN);
	 print(14,1," FIELD ");
	 print(2,2,"Enter file ==>");
	 key=line_editor(18,2,57,C_MAIN,file,TRUE,TRUE);
	 if(key==ESC || key==F10) break;
	 strcat(file,".CAL");
	 if((f_field=fopen(file,"rb"))==NULL) file_not_good(3);
    } while(f_field==NULL);

    if(key!=ESC && key!=F10)
    {
	 strcpy(ffield,file);
	 fread(&dfield_in[1],sizeof(double),N+1,f_field);
	 fclose(f_field);
	 f_sample=sample=dfield_in[N+1];
	 if(f_sample==0.0) { file_not_good(9); f_sample=t_sample; }
	 for(i=1;i<=N;i++) dfield[i]=dfield_in[i];
	 min_cr=(FRAMP*2.*f_sample)/(N*t_sample);
	 if(min_cr<0.01) min_cr=0.01;
	 if(c_ratio<min_cr) c_ratio=min_cr;
    }
    win_delete(wfield);
  }
  else
  {
    if((f_field=fopen(ffield,"rb"))==NULL)
    {
	 file_not_good(3);
	 strcpy(ffield,"UNTITLED");
    }
    else 
    {
	 fread(&dfield_in[1],sizeof(double),N+1,f_field);
	 fclose(f_field);
    }
    f_sample=dfield_in[N+1];
    if(f_sample==0.0) { file_not_good(9); f_sample=t_sample; }
    for(i=1;i<=N;i++) dfield[i]=dfield_in[i];
    min_cr=(FRAMP*2.*f_sample)/(N*t_sample);
    if(min_cr<0.01) min_cr=0.01;
    if(c_ratio<min_cr) c_ratio=min_cr;
  }
  if(option==TRUE && key!=ESC && key!=F10) {if(list) win_delete(list); list_file();}
  win_redraw_all();
  if(key==F10)
  {
    check_quit();
    import_field(1);
  }
}

/*************************************************************************/
void import_tran(int option)         /* input .TRN file */
{
  char file[80];
  int key,i,k;
  FILE *f_tran;

  if(option)
  {
    if((strcmp(ftran,"UNTITLED"))!=0)
    {
	 strcpy(file,ftran);
	 k=strlen(file);
	 file[k-4]='\0';
    }
    else strcpy(file,"\0");

    do
    {
	 win_cls(help);
	 scr.current=LGREEN;
	 win_print(help,3,1,"Enter name of a previously saved transmission file.");
	 win_print(help,3,2,"Do not include file extension.");
	 win_print(help,3,3,"File should be saved with '.TRN' extension.");
	 key_win(2);
	 trans=win_make(22,9,37,2,STD_FRAME,"",YELLOW,C_MAIN);
	 print(14,1," TRAN ");
	 print(2,2,"Enter file ==>");
	 key=line_editor(18,2,57,C_MAIN,file,TRUE,TRUE);
	 if(key==ESC || key==F10) break;
	 strcat(file,".TRN");
	 if((f_tran=fopen(file,"rb"))==NULL) file_not_good(4);
    } while(f_tran==NULL);

    if(key!=ESC && key!=F10)
    {
	 strcpy(ftran,file);
	 fread(&dtdata_in[1],sizeof(double),N+1,f_tran);
	 fclose(f_tran);
	 for(i=1;i<=N;i++) dtdata[i]=dtdata_in[i];
	 t_sample=dtdata_in[N+1];
	 if(t_sample==0.0) { file_not_good(8); t_sample=0.2; }
	 c_ratio=1.0;
	 min_cr=(FRAMP*2.*f_sample)/(N*t_sample);
	 if(min_cr<0.01) min_cr=0.01;
    }
    win_delete(trans);
  }
  else
  {
    if((f_tran=fopen(ftran,"rb"))==NULL)
    {
	 file_not_good(4);
	 strcpy(ftran,"UNTITLED");
    }
    else
    {
	 fread(&dtdata_in[1],sizeof(double),N+1,f_tran);
	 fclose(f_tran);
	 for(i=1;i<=N+1;i++) dtdata[i]=dtdata_in[i];
	 t_sample=dtdata_in[N+1];
	 if(t_sample==0.0) { file_not_good(8); t_sample=0.2; }
	 if (f_sample==0.0) f_sample=t_sample;
	 if (sample==0.0) sample=t_sample;
	 min_cr=(FRAMP*2.*f_sample)/(N*t_sample);
	 if(min_cr<0.01) min_cr=0.01;
    }
  }
  if(option==TRUE && key!=ESC && key!=F10) {if(list) win_delete(list); list_file();}
  win_redraw_all();
  if(key==F10)
  {
    check_quit();
    import_tran(1);
  }
}

/*************************************************************************/
int display_gain(int g_mode)  /* calculate & remove range-gain */
{
  unsigned char cgdata[N+1];
  char lab[80];
  int key,i,j,k,a,b,e,f,bytesin,scan_cnt,hw;
  double rgdata[N+1],maxf[N+1],minf[N+1],trend[N+1];
  double mindum,maxdum;
  FILE *gain_file,*tmp_file;

  gain_file=fopen(fgain,"rb");
  scan_cnt=0;
  for(i=0;i<N;i++) { rgdata[i]=0.; maxf[i]=0.; minf[i]=0.; }
  graph_setup("Range Gain Calibration Records And Interpolated Envelope",g_mode);

  do
  {
    bytesin=fread(cgdata,1,N,gain_file);
    scan_cnt++; 
    if ((bytesin<N) || (scan_cnt>8)) break;
    for (i=0;i<N;i++) { rgdata[i]=(double)cgdata[i]; trend[i]=0.;}
    /**************** Calculate low frequency trends ********/
    for (i=0;i<N;i++)
    {
	 hw=((int)(.05*i))*((int)(.05*i))+1; a=i-hw; b=i+hw;
	 if (i>400) { trend[i]=trend[i-1]; } 
	 else 
	 {
	   if (a<0) a=0;
	   if (b>N-1) b=N-1;
	   for (j=a;j<=b;j++) trend[i]+=rgdata[j]; 
	   trend[i]/=(double)(b-a+1); 
	 }
    }
    for (i=0;i<N;i++) rgdata[i]-=trend[i]; 
    for (i=0;i<LITER/2;i++) rgdata[i]=0.;
    /**************** Fill min and max function contributions *****/
    for (i=LITER;i<N;i++)
    {
	 if (rgdata[i-1]<rgdata[i] && rgdata[i+1]<rgdata[i] && rgdata[i]>maxf[i] && \
		 (rgdata[i]+trend[i])<255.) maxf[i]=rgdata[i];
	 if (rgdata[i-1]>rgdata[i] && rgdata[i+1]>rgdata[i] && rgdata[i]<minf[i] && \
		 (rgdata[i]+trend[i])>0.) minf[i]=rgdata[i];
    }
    /**************** Plot rgdata *******************/
    for(i=0;i<N;i++) rgdata[i]+=127.;
    pen(13);                     /* magenta */
    for(i=1;i<N;i++) plot((double)i,rgdata[i-1],-1-(i==1));

  } while (TRUE);

  /***********************************************************************/
  fclose(gain_file);
  /************ Zero maxf and minf if not monotonic *****************/
  for (i=0;i<=LITER;i++) { maxf[i]=1.; minf[i]=-1.; }
  for (i=LITER;i<N;i++) 
  {
    if (maxf[i]!=0.0) 
    { 
	 for (j=i+1;j<N;j++) { if (maxf[j]<maxf[i]) maxf[j]=0.0; } 
    }
    if (minf[i]!=0.0) 
    { 
	 for (k=i+1;k<N;k++) { if (minf[k]>minf[i]) minf[k]=0.0; } 
    }
  }
  /*********** Interpolate minf and maxf where zero *************/
  a=0; e=0;
  for (i=LITER;i<N;i++) 
  {
    if (maxf[i]!=0.0) 
    { 
	 b=i; 
	 for(j=a+1;j<b;j++)
	 { 
	   maxf[j]=maxf[a]+((maxf[b]-maxf[a])*((double)(j-a)/(double)(b-a))); 
	 }
	 a=b; 
    }
    if (minf[i]!=0.0) 
    { 
	 f=i; 
	 for(k=e+1;k<f;k++)
	 { 
	   minf[k]=minf[e]+((minf[f]-minf[e])*((double)(k-e)/(double)(f-e))); 
	 }
	 e=f; 
    }
  }                             
  /*********** Extrapolate minf and maxf if necessary ***************/    
  if (b<f)
  {
    b=f;
    for(j=a+1;j<=b;j++)
    { 
	 maxf[j]=maxf[a]-((minf[b]-minf[a])*((double)(j-a)/(double)(b-a))); 
    }
    a=b;
  } 
  if (f<b)
  {
    f=b;
    for(k=e+1;k<=f;k++)
    { 
	 minf[k]=minf[e]-((maxf[f]-maxf[e])*((double)(k-e)/(double)(f-e))); 
    }
    e=f;
  }  
  if (b<N-1)
  {
    b=N-1;
    for(j=a+1;j<=b;j++) 
    {
	 if ((2*a-N)>0) 
	 {
	   maxf[j]=maxf[a]+(maxf[a+j-b]-maxf[2*a-b]);
	 } 
	 else 
	 {
	   maxf[j]=maxf[a];
	 }
    }
  } 
  if (f<N-1)
  {
    f=N-1;
    for(k=e+1;k<=f;k++) 
    {
	 if ((2*a-N)>0) 
	 {
	   minf[k]=minf[e]+(minf[e+k-f]-minf[2*e-f]);
	 } 
	 else 
	 {
	   minf[k]=minf[e];
	 }
    }
  }   
  /************* smooth minf and maxf ************************/
  smooth_data(maxf,N,W);
  smooth_data(minf,N,W);
  for(i=0;i<N;i++) { maxf[i]+=127.; minf[i]+=127.; }

  /*********************** Display gain envelope *************************/
  pen(15);                       /* white */
  plot(0.,maxf[0],-2);
  for(i=1;i<N;i++) plot((double)i,maxf[i],-1);
  plot(0.,minf[0],-2);
  for(i=1;i<N;i++) plot((double)i,minf[i],-1);
  /*********** say when done ***************/
  viewport(10.,125.,0.,10.);
  window(0.,100.,0.,100.);
  csize(0.8); ldir(0.); lorg(2); pen(14); /* yellow */
  label(15.,30.,"Completed - Press a key to continue");
  /*****************************************/
  getch();
  plotter_off(); 
  remove_gain(maxf,minf,g_mode);
  win_redraw_all();
  scr.current=WHITE;
  win_delete(work);
  work=win_make(2,3,78,16,DBL_FRAME,"",WHITE,C_MAIN);
  scr.current=WHITE;
  cursor(NO_CURSOR);
  print(5,2,"     The next screen will allow the user to eliminate data");
  print(5,3,"at the beginning of the field scan. This may be necessary if");
  print(5,4,"the field scan contains a direct wave pulse transmitted between");
  print(5,5,"transmitting and receiving antennas in bistatic mode.");
  print(5,6,"The direct wave pulse will not be forward modeled in this program,");
  print(5,7,"so it is recommended that it be deleted for comparison purposes");
  print(5,8,"This final processed field scan will be stored with the ");
  sprintf(lab,"file name %s.CAL.",fraw);
  print(5,9,lab);
  print(5,10,"     To eliminate data at the beginning of the field scan,");
  print(5,11,"use  and  keys to align blue line.  Press <Enter>");
  print(5,12,"to eliminate all data to the left of the blue line.");
  print(5,13,"Press <Space> to rescale plot.");
  print(5,14,"Press <ESC> to backup.");
  key_win(0);
  key=getch();
  if(key==0 || key==0xE0) key=getch();
  if(key==ESC) return(ESC);
  eliminate_field(g_mode);
  win_delete(work);
  work=win_make(2,3,78,16,DBL_FRAME,"",WHITE,C_MAIN);
  win_redraw_all();
  cls();
  /******* Write final processed field data to file.CAL ********************/
  strcpy(ffield,fraw);
  strcat(ffield,".CAL");
  tmp_file=fopen(ffield,"wb");
  dfield[N+1]=f_sample;
  fwrite(&dfield[1],sizeof(double),N+1,tmp_file);
  fclose(tmp_file);
  for(i=1;i<=N;i++) dfield_in[i]=dfield[i];
  return(key);
}

/*************************************************************************/
void smooth_data(double *data, int n, int w)    /* smooths by window w */
{
  int i,j,hw;
  double sumwin=0.0, smoo_gate[N/2];

  hw=(w-1)/2;  /* set half-window */
  for(j=0;j<w;j++) smoo_gate[j]=data[j];  /* load smoothing gate */

  /*************** smooth data to middle of gate *****************/
  for(i=0;i<=hw;i++) { sumwin+=data[i]; data[i]=sumwin/(i+1); } 

  /**********finish loading first sum of smoothing gate values ******/
  for(j=hw+1;j<w;j++) sumwin+=smoo_gate[j];

  /************* move smoothing gate through data **************/
  for(i=hw+1;i<n-hw;i++)
  {
    sumwin-=smoo_gate[0]; /**** drop 1st gate value from sum ****/
    for(j=0;j<w-1;j++) smoo_gate[j]=smoo_gate[j+1]; /*** roll gate ***/
    smoo_gate[w-1]=data[i+hw];  /**** pick up new gate value ****/
    sumwin+=smoo_gate[w-1];  /*** add new gate value to sum ***/
    data[i]=sumwin/w;  /*** output average over gate ****/
  }

  /************** smooth to end of data ***********************/
  for(i=n-hw;i<n;i++) 
  {
    sumwin-=(data[i-hw-1]+data[i-hw]); 
    w-=2; hw-=1;
    data[i]=sumwin/w;
  }
}

/*************************************************************************/
void remove_gain(double *maxf, double *minf, int g_mode)  /* removes gain */
{
  int i,j,m;
  double raw_no_trend[N+1],mn,mx,sum,bias;

  mn=9999.; mx=-9999.;
  for (i=0;i<N;i++) raw_no_trend[i+1]=(double)craw[i]; 
  /**************** Remove low frequency trends ********/
  realft(raw_no_trend,N/2,1);
  m=2*(FRAMP-1);
  for(i=0;i<m;i++) raw_no_trend[i]=0.0;
  for(i=m*0.5;i<m;i++) 
  {
    raw_no_trend[2*i]*=0.5+(0.5*cos(2.*PI*i/m));
    raw_no_trend[(2*i)+1]*=0.5+(0.5*cos(2.*PI*i/m));
  }
  realft(raw_no_trend,N/2,-1);
  for (i=1;i<=N;i++) raw_no_trend[i]/=((double)N/2.); 
  /************** scale around 127 ****************/
  sum=0.0;
  for(i=1;i<=N;i++) sum+=raw_no_trend[i];
  bias=sum/(double)(N);
  for(i=1;i<=N;i++) raw_no_trend[i]-=(bias-127);
  /***************** build dfield as craw w/o gain *******/
  for(i=1;i<=N;i++)
  {
    if(raw_no_trend[i]>127. && maxf[i]>128.)
    dfield[i]=127.+((raw_no_trend[i]-127.)/(maxf[i]-127.));
    else if(raw_no_trend[i]<127. && minf[i]<127.)
    dfield[i]=127.-((raw_no_trend[i]-127.)/(minf[i]-127.));
    else dfield[i]=raw_no_trend[i];
  }
  /****************** zero 1st part and center dfield around 0 ***********/
  for(i=1;i<LITER;i++) dfield[i]=0.;
  for(i=LITER;i<=N;i++) dfield[i]-=127.;
}

/*************************************************************************/
void eliminate_field(int g_mode) /* zero out direct arrivals */
{
  int i,key,inc=1,plot_bar=1;
  double mn,mx,dif,tmp_field[N+1];
  char str[80];

  plotter_is(g_mode,"CRT");
  viewport(10.,120.,10.,95.);
  window(0.,100.,0.,100.);
  pen(12); /* red */
  frame();
  viewport(10.,120.,95.,100.);
  window(0.,100.,0.,100.);
  csize(1.); pen(15); /* white */
  lorg(5);
  label(50.,50.,"Elimination Of Field Data At Start Of Scan");
  viewport(10.,120.,0.,10.);
  window(0.,100.,0.,100.);
  lorg(2);
  label(10.,60.,"Align blue bar and press <Return>.");
  label(10.,30.,"Press <Space> to rescale plot.");
  lorg(4);
  label(0.,90.,"0");
  label(100.,90.,"512");
  viewport(12.,118.,10.,95.);
  for(i=1;i<=N;i++) tmp_field[i]=dfield[i];
  tmp_field[0]=tmp_field[1];

  do
  {
    pen(0);
    plot(1.,tmp_field[1],-2);
    for(i=2;i<=N;i++) plot((double)i,tmp_field[i],-1);
    mn=9999.;                    /* Determine window limits */
    mx=(-9999.);
    for(i=LITER;i<N;i++)
    {
	 mn=MIN(mn,tmp_field[i]);
	 mx=MAX(mx,tmp_field[i]);
    }
    dif = 0.05*(mx-mn);
    mn -= dif;
    mx += dif;
    window(0.,(double)N+1.,mn,mx);
    clip(); pen(15); /* white */
    for(i=2;i<=N;i++) plot((double)i,tmp_field[i],-1-(i==2));
    if(plot_bar) 
    {
	 pen(9); plot(1.,dfield[1]-dif,-2);
	 plot(1.,dfield[1]+dif,-1);
    }
    lorg(2);

    do
    { 
	 key=getch();
	 if(key==0 || key==0xE0) key=getch();
	 if(key==LEFT && inc>1)
	 {
	   pen(0);
	   plot((double)(inc),dfield[inc]-dif,-2);
	   plot((double)(inc),dfield[inc]+dif,-1);
	   plot((double)(inc-1),tmp_field[inc-1],-2);
	   plot((double)inc,tmp_field[inc],-1);
	   plot((double)(inc+1),tmp_field[inc+1],-1);
	   pen(9);                  /* blue */
	   plot((double)(inc-1),dfield[inc-1]-dif,-2);
	   plot((double)(inc-1),dfield[inc-1]+dif,-1);
	   tmp_field[inc]=dfield[inc]; tmp_field[inc-1]=0.;
	   pen(15);                 /* white */
	   plot((double)(inc-2),tmp_field[inc-2],-2);
	   plot((double)(inc-1),tmp_field[inc-1],-1);
	   plot((double)inc,tmp_field[inc],-1);
	   plot((double)(inc+1),tmp_field[inc+1],-1);
	   plot((double)(inc+2),tmp_field[inc+2],-1);
	   inc--;
	 }
	 if(key==RIGHT && inc<N)
	 {
	   pen(0);
	   plot((double)(inc),dfield[inc]-dif,-2);
	   plot((double)(inc),dfield[inc]+dif,-1);
	   plot((double)inc,tmp_field[inc],-2);
	   plot((double)(inc+1),tmp_field[inc+1],-1);
	   plot((double)(inc+2),tmp_field[inc+2],-1);
	   pen(9);                  /* blue */
	   plot((double)(inc+1),dfield[inc+1]-dif,-2);
	   plot((double)(inc+1),dfield[inc+1]+dif,-1);
	   tmp_field[inc]=tmp_field[inc+1]=0.;
	   pen(15);                 /* white */
	   plot((double)(inc-1),tmp_field[inc-1],-2);
	   plot((double)inc,tmp_field[inc],-1);
	   plot((double)(inc+1),tmp_field[inc+1],-1);
	   plot((double)(inc+2),tmp_field[inc+2],-1);
	   inc++;
	 }
	 pen(0);
	 label(10.,(mn+mx)/2.,str);
	 sprintf(str,"  %d", inc);
	 pen(14);                   /* yellow */
	 label(10.,(mn+mx)/2.,str);
    } while(key!=RETURN && key!=SPACE);

    plot_bar=0;
  } while(key!=RETURN);

  unclip();
  plotter_off();
  for(i=1;i<=N;i++) dfield[i]=tmp_field[i];
}

/*************************************************************************/
void time_calibrate(int g_mode)  /* find f_sample from .TC file */
{
  unsigned char ctime[N+1];
  char sample_display[80],calibrate_display[80],autocount[80];
  int i,j=0,k,pkpnts1[N/2],pkpnts2[N/2],pkpnts3[N/2],cnt;
  double mean1,mean2,mean3,stdev1,stdev2,stdev3;
  double level,sum,stack[N+1],ostack[N+1];
  double digi,dev,midamp,maxamp=-1e32,minamp=1e32;
  FILE *time_file;

  time_file=fopen(ftime,"rb");
  for(i=0;i<N;i++) stack[i]=0.;
  graph_setup("Time Calibration To Determine Sample Rate",g_mode);

  do
  {
    k=fread(ctime,1,N,time_file);
    if(k<N) break;
    j++;
    for(i=0;i<N;i++) stack[i] += (double)ctime[i];    /* stack time scans */
  } while(TRUE);

  fclose(time_file);
  for(i=0;i<N;i++) stack[i] /= (double)j;   /* final stacked time scan for calculation */
  for (i=0;i<N;i++) 
  { 
    ostack[i]=stack[i]; 
    if (stack[i]<minamp) minamp=stack[i]; 
    if (stack[i]>maxamp) maxamp=stack[i]; 
  }
  midamp=(maxamp+minamp)*.5;
  for (i=0;i<N;i++) stack[i]=stack[i]-midamp;
  level=midamp-20.;
  /***** count the number of significant downslopes *******************/
  cnt=count(stack,N);
  if(cnt==0) { file_not_good(7); return; }

  /************** next 2 lines for debugging if program fails *********/
  /*  sprintf(autocount,"Auto count is %d.",cnt);  
	 label((double)100.,60.,autocount); */

  pick(stack,N,cnt,0,pkpnts1);   /* divides data into cnt regions and picks min or max of each */
  mean1=mean(pkpnts1,cnt);       /* finds mean # of points between picks */
  if(cnt>2) stdev1=stddev(pkpnts1,cnt,mean1);    /* finds stddev of points between picks */
  else stdev1=0.0;
  derivative(stack,9,N);         /* creates derivative of data with 9 pnt smoother */
  pick(stack,N,cnt,1,pkpnts2);   /* picks cnt maximums of derivative */
  mean2=mean(pkpnts2,cnt);
  if(cnt>2) stdev2=stddev(pkpnts2,cnt,mean2);
  else stdev2=0.0;
  derivative(stack,9,N);         /* another 9-pnt derivative (now 2nd deriv) */
  pick(stack,N,cnt,1,pkpnts3);   /* picks cnt maximums of 2nd deriv */
  mean3=mean(pkpnts3,cnt);
  if(cnt>2) stdev3=stddev(pkpnts3,cnt,mean3);
  else stdev3=0.0;
  if ((stdev2<stdev1) && (stdev2!=0.0)) 
  { 
    mean1=mean2; 
    stdev1=stdev2; 
    for (i=0;i<cnt;i++) pkpnts1[i]=pkpnts2[i]; 
  }
  if ((stdev3<stdev1) && (stdev3!=0.0)) 
  { 
    mean1=mean3; 
    stdev1=stdev3; 
    for (i=0;i<cnt;i++) pkpnts1[i]=pkpnts3[i]; 
  }                              /* finds lowest stddev */
  for (i=0;i<N;i++) stack[i]=ostack[i];     /* returns data to original */
  digi=10./mean1;
  dev=digi-(10./(mean1+stdev1));
  j=cnt;
  /********************* Display calibration information *****************/
  pen(15);                       /* white */
  for(i=1; i<N; i++) plot((double)i,stack[i],-1-(i==1));
  pen(12);                       /* red */
  lorg(5); ldir(0);
  for(i=0; i<j; i++) label((double)pkpnts1[i],level,"+");
  if(j>4) {j=j/2+1;} else {if(j==2) j=0;}
  pen(14);                       /* yellow */
  plot((double)pkpnts1[j],level-23.,-2);
  plot((double)pkpnts1[j],level-17.,-1);
  plot((double)pkpnts1[j],level-20.,-1);    /* plot 10-nsec scale bar */
  plot((double)pkpnts1[j]-mean1,level-20.,-1);
  plot((double)pkpnts1[j]-mean1,level-23.,-1);
  plot((double)pkpnts1[j]-mean1,level-17.,-1);
  sprintf(calibrate_display,"%3.2f pts/10 nsec",mean1);
  label((double)pkpnts1[j]-mean1/2.,level-30.,calibrate_display);
  pen(15);                       /* white */
  viewport(10.,133.,0.,10.);
  window(0.,100.,0.,100.);
  csize(0.75);
  sprintf(calibrate_display,"%3.2f pts/10-ns [std.dev.=%3.4f]",mean1,stdev1);
  label(50.,66.,calibrate_display);
  sprintf(sample_display,"%3.4f ns/pt [std.dev.=%3.4f]",digi,dev);
  label(50.,33.,sample_display);
  getch();
  plotter_off();
  sample=f_sample=digi;
  min_cr=(FRAMP*2.*f_sample)/(N*t_sample);
  if(min_cr<0.01) min_cr=0.01;
  if(c_ratio<min_cr) c_ratio=min_cr;
}

/*************************************************************************/
void derivative(double *data,int width,int n)    /* returns deriv smoothed by width */
{  
  int i,j,k,hw;
  double dw=0.0,ddata[N+1],newdata;

  for (i=0;i<n;i++) ddata[i]=data[i];
  hw=(width-1)/2;
  for (i=1;i<=hw;i++) dw+=(double)(i*i);
  dw*=2.;

  for (i=0;i<n;i++)
  {
    newdata=0.0;
    for (j=(-hw);j<=hw;j++)  
    {
	 k=i+j;
	 if (k<0) k=0;
	 if (k>(n-1)) k=n-1; 
	 newdata+=(double)(j)*ddata[k];
    }
    data[i]=newdata/dw;
  }
}

/*************************************************************************/
void pick(double *data, int n, int np, int hilo, int *pkpnts)
{ 
  int i,k,pwin,hpwin;
  int p[N/2];
  double ex;

  if (hilo!=0) { ex=1.;} else {ex=-1.;}
  pwin=(n/np);
  hpwin=0;
  p[0]=n-1;
  for (i=0;i<20;i++) data[i]=0.0;
  for (i=1;i<=np;i++) p[i]=0;
  if (np>3)
  {
    for (k=1;k<=np;k++)
    {
	 p[k]=0;
	 for (i=p[k-1]-hpwin;i>=p[k-1]-pwin;i--)
	 {
	   if ( (data[i]*ex)>(data[p[k]]*ex) ) p[k]=i;
	 }
	 hpwin=(pwin/2);
    }
  }
  else
  {
    for (k=1;k<=np;k++)
    {
	 p[k]=0;
	 for (i=(n-1)-(pwin*(k-1));i>(n-1)-(pwin*k);i--)
	 {
	   if ( (data[i]*ex)>(data[p[k]]*ex) ) p[k]=i;
	 }
    }
  }
  for (k=1;k<=np;k++) pkpnts[k-1]=p[k];
}

/*************************************************************************/
int count(double *data, int n)   /* counts regional minima in data */
{
  int cnt=0,i;
  double range,minamp=1e32,maxamp=-1e32,threshold=0;

  for (i=21;i<n;i++)
  { 
    if (data[i]<minamp) minamp=data[i]; 
    if (data[i]>maxamp) maxamp=data[i]; 
  }
  range=maxamp-minamp;
  if (range==0.0) return(0);
  threshold=minamp+(.4*range);

  do 
  {
    cnt=0;
    for (i=21;i<n;i++) 
    { 
	 if ((data[i]<threshold)&&(data[i-1]>threshold)) cnt++; 
    }
    threshold+=.02*range;
  } while (cnt<2);

  if (cnt>9) cnt-=2;
  return(cnt);
}

/*************************************************************************/
double mean(int *pkpnts, int cnt)
{
  int cntm1,i;
  double mean1=0.0,pkint[N/2];

  cntm1=cnt-1;
  for (i=0;i<cntm1;i++) pkint[i]=pkpnts[i]-pkpnts[i+1];
  for (i=0;i<cntm1;i++) mean1+=pkint[i];
  mean1/=(double)cntm1;
  return(mean1);
}

/*************************************************************************/
double stddev(int *pkpnts, int cnt, double mn)
{
  int cntm1,cntm2,i;
  double stdev1=0.0,pkint[N/2];

  cntm1=cnt-1;
  cntm2=cnt-2;
  if (cntm2==0) cntm2=1;
  for (i=0;i<cntm1;i++) pkint[i]=pkpnts[i]-pkpnts[i+1];
  for (i=0;i<cntm1;i++) stdev1+=(double)((pkint[i]-mn)*(pkint[i]-mn));
  stdev1/=(double)cntm2;
  stdev1=sqrt(stdev1);
  return(stdev1);
}

/*************************************************************************/
void graph_setup(char *text,int g_mode)      /*  Setup gain display and stack graphs */
{
  int i;
  plotter_is(g_mode,"CRT");
  pen(15); lorg(5); csize(1.); /* white */
  viewport(10.,125.,95.,100.);
  window(0.,100.,0.,100.);
  label(50.,50.,text);
  viewport(0.,10.,10.,95.);
  window(0.,100.,0.,100.);
  csize(1.7); ldir(PI/2.);
  label(75.,83.,"255");
  label(75.,13.,"0");
  viewport(10.,125.,0.,10.);
  window(0.,100.,0.,100.);
  csize(1.2); ldir(0.); lorg(4);
  label(0.,90.,"0");
  label(100.,90.,"512");
  pen(3); /* cyan */
  viewport(10.,125.,10.,95.);
  window(0.,512.,-50.,305.);
  frame();
  for(i=1;i<6;i++)
  {
    plot(0.,50.*i,-2);
    plot(512.,50.*i,-1);
    plot(100.*i,0.,-2);
    plot(100.*i,255.,-1);
  }
  pen(7); /* dull white */
  plot(0.,255.,-2); plot(512.,255.,-1);
  plot(0.,0.,-2); plot(512.,0.,-1);
  pen(12); /* red */
  plot(0.,127.,-2);
  plot(511.,127.,-1);
}

/*************************************************************************/
void realft(double *data,int n,int isign)  /* FFT from Numerical Recipes in C */
{
  int i,i1,i2,i3,i4,n2p3;
  double c1=0.5,c2,h1r,h1i,h2r,h2i;
  double wr,wi,wpr,wpi,wtemp,theta;
  void four1();

  theta=PI/(double) n;
  if (isign == 1)
  {
    c2 = -0.5;
    four1(data,n,1);
  }
  else
  {
    c2=0.5;
    theta = -theta;
  }
  wtemp=sin(0.5*theta);
  wpr = -2.0*wtemp*wtemp;
  wpi=sin(theta);
  wr=1.0+wpr;
  wi=wpi;
  n2p3=2*n+3;
  for (i=2;i<=n/2;i++)
  {
    i4=1+(i3=n2p3-(i2=1+(i1=i+i-1)));
    h1r=c1*(data[i1]+data[i3]);
    h1i=c1*(data[i2]-data[i4]);
    h2r = -c2*(data[i2]+data[i4]);
    h2i=c2*(data[i1]-data[i3]);
    data[i1]=h1r+wr*h2r-wi*h2i;
    data[i2]=h1i+wr*h2i+wi*h2r;
    data[i3]=h1r-wr*h2r+wi*h2i;
    data[i4] = -h1i+wr*h2i+wi*h2r;
    wr=(wtemp=wr)*wpr-wi*wpi+wr;
    wi=wi*wpr+wtemp*wpi+wi;
  }
  if (isign == 1)
  {
    data[1] = (h1r=data[1])+data[2];
    data[2] = h1r-data[2];
  }
  else
  {
    data[1]=c1*((h1r=data[1])+data[2]);
    data[2]=c1*(h1r-data[2]);
    four1(data,n,-1);
  }
}

/*************************************************************************/
void four1(double *data,int nn,int isign)  /* FFT from Numerical Recipes in C */ 
{
  int n,mmax,m,j,istep,i;
  double wtemp,wr,wpr,wpi,wi,theta;
  double tempr,tempi;

  n=nn << 1;
  j=1;
  for (i=1;i<n;i+=2)
  {
    if (j > i)
    {
	 SWAP(data[j],data[i]);
	 SWAP(data[j+1],data[i+1]);
    }
    m=n >> 1;

    while (m >= 2 && j > m)
    {
	 j -= m;
	 m >>= 1;
    }
  
    j += m;
  }
  mmax=2;

  while (n > mmax)
  {
    istep=2*mmax;
    theta=2*PI/(isign*mmax);
    wtemp=sin(0.5*theta);
    wpr = -2.0*wtemp*wtemp;
    wpi=sin(theta);
    wr=1.0;
    wi=0.0;
    for (m=1;m<mmax;m+=2)
    {
	 for (i=m;i<=n;i+=istep)
	 {
	   j=i+mmax;
	   tempr=wr*data[j]-wi*data[j+1];
	   tempi=wr*data[j+1]+wi*data[j];
	   data[j]=data[i]-tempr;
	   data[j+1]=data[i+1]-tempi;
	   data[i] += tempr;
	   data[i+1] += tempi;
	 }
	 wr=(wtemp=wr)*wpr-wi*wpi+wr;
	 wi=wi*wpr+wtemp*wpi+wi;
    }
    mmax=istep;
  }

}

/*************************************************************************/
void spline(double *x,double *y,int n,double yp1,double ypn,double *y2)
{
  int i,k;
  double pp,qn,sig,un,u[N];

  if (yp1 > 0.99e30) y2[1]=u[1]=0.0;
  else 
  {
    y2[1] = -0.5;
    u[1]=(3.0/(x[2]-x[1]))*((y[2]-y[1])/(x[2]-x[1])-yp1);
  }
  for (i=2;i<=n-1;i++)
  {
    sig=(x[i]-x[i-1])/(x[i+1]-x[i-1]);
    pp=sig*y2[i-1]+2.0;
    y2[i]=(sig-1.0)/pp;
    u[i]=(y[i+1]-y[i])/(x[i+1]-x[i]) - (y[i]-y[i-1])/(x[i]-x[i-1]);
    u[i]=(6.0*u[i]/(x[i+1]-x[i-1])-sig*u[i-1])/pp;
  }
  if (ypn > 0.99e30) qn=un=0.0;
  else
  {
    qn=0.5;
    un=(3.0/(x[n]-x[n-1]))*(ypn-(y[n]-y[n-1])/(x[n]-x[n-1]));
  }
  y2[n]=(un-qn*u[n-1])/(qn*y2[n-1]+1.0);
  for (k=n-1;k>=1;k--) y2[k]=y2[k]*y2[k+1]+u[k];
}

/*************************************************************************/
void splint(double *xa,double *ya,double *y2a,int n,double x,double *y)
{
  int klo,khi,k;
  double h,b,a;

  klo=1;
  khi=n;

  while (khi-klo > 1)
  {
    k=(khi+klo) >> 1;
    if (xa[k] > x) khi=k;
    else klo=k;
  }

  h=xa[khi]-xa[klo];
  if (h == 0.0) printf("Bad XA input to routine SPLINT");
  a=(xa[khi]-x)/h;
  b=(x-xa[klo])/h;
  *y=a*ya[klo]+b*ya[khi]+((a*a*a-a)*y2a[klo]+(b*b*b-b)*y2a[khi])*(h*h)/6.0;
}

